/**
 ****************************************************************************
 * @file	D_Driver.c
 * @brief	Motor vector control driver Source File
 * @version	V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */
#include <stdlib.h>
#include "ipdefine.h"
#include "mcuip_drv.h"
#include "D_Para.h"
#include "E_Sub.h"

#include "ipdrv_enc.h"
#include "ipdrv_adc.h"


#define DEFINE_APP
#include "D_Driver.h"
#undef	DEFINE_APP


/*===================================================================*
	  Local Variable Definition
 *===================================================================*/
/*===================================================================*
	  Local Proto Type Definition
 *===================================================================*/
static int32_t Limit_sub(int64_t _indata, int32_t _limlo, int32_t _limup);


/**
 *********************************************************************************************
 * @brief		H_Encoder
 *				Adjust Z-phase, 0deg of electrical-angle
 * @param		vector_t* const			_motor	The structure address for motor vector control.
 * @param		TSB_EN_TypeDef * const	ENx		Select the encoder channel.
 *
 * @return		none
 *********************************************************************************************
 */
#if defined(__USE_ENCODER_CH0) || defined(__USE_ENCODER_CH1)
void H_Encoder(vector_t* const _motor, TSB_EN_TypeDef* const ENx)
{
	ENC_FlagStatus	EncFlg;

	EncFlg = ENC_GetENCFlag(ENx);

	/* Phase Z is detected? */
	if (EncFlg.Bit.ZPhaseDetectFlag == SET)
	{
		/* Get Encoder counter value */
		_motor->drv.EnCnt = ENC_GetCounterValue(ENx);
	}

	/* Calculation theta(electrical-angle) from encoder counter */
	_motor->drv.theta_enc = (uint32_t)(((uint64_t)_motor->drv.EnCnt * _motor->para.enc.pls2theta) + _motor->para.enc.deg_adjust);

	if (EncFlg.Bit.RotationDirection == 1)
	{
		/* CW */
		if (_motor->drv.EnCnt >= _motor->drv.EnCnt1)
		{
			_motor->drv.EnCnt_dev = (q15_t)(_motor->drv.EnCnt - _motor->drv.EnCnt1);
		}
		else
		{
			_motor->drv.EnCnt_dev = (q15_t)(_motor->para.enc.plsnum + _motor->drv.EnCnt - _motor->drv.EnCnt1);
		}
	}
	else
	{
		/* CCW */
		if (_motor->drv.EnCnt1 >= _motor->drv.EnCnt)
		{
			_motor->drv.EnCnt_dev = (q15_t)((_motor->drv.EnCnt1 - _motor->drv.EnCnt) * (-1));
		}
		else
		{
			_motor->drv.EnCnt_dev = (q15_t)((_motor->para.enc.plsnum + _motor->drv.EnCnt1 - _motor->drv.EnCnt) * (-1));
		}
	}

	_motor->drv.EnCnt1 = _motor->drv.EnCnt;

	/* Cal omega(electrical-angle) from encoder counter deviation */
	_motor->drv.omega_enc_raw = (q15_t)(_motor->drv.EnCnt_dev * _motor->para.enc.pls2omega);
	_motor->drv.omega_enc_ave.word = calc_ave(_motor->drv.omega_enc_ave.word, _motor->drv.omega_enc_raw << 16 , cOMEGAENC_AVE);
	_motor->drv.omega_enc = _motor->drv.omega_enc_ave.half[1];

	if (_motor->drv.command.encoder == SET)
	{
		_motor->drv.omega.word = _motor->drv.omega_enc << 16;

		if (_motor->drv.vector_cmd.F_vcomm_theta == SET)
		{
			if (_motor->drv.vector_cmd.F_vcomm_omega == SET)
			{
				if (EncFlg.Bit.ZPhaseDetectFlag == 1)
				{
					/* Phase Z is detected */
					_motor->drv.theta.word = _motor->drv.theta_enc;
				}
				else
				{
					/* THETAn = THETAn-1 + OMEGA * t */
					/* theta = (omega * 2^16 * HZ_MAX) * Control period */
					_motor->drv.theta.word += ((_motor->drv.omega_com.half[1] * _motor->para.enc.ctrlprd) << 1);
				}
			}
			else
			{
				_motor->drv.theta.word += (_motor->drv.omega_com.half[1] * _motor->para.enc.ctrlprd) << 1;
			}
		}
		else
		{
			if (EncFlg.Bit.ZPhaseDetectFlag == 1)
			{
				/* Phase Z is detected */
				_motor->drv.theta.word = _motor->drv.theta_enc;
			}
			else
			{
				_motor->drv.theta.word = (_motor->drv.theta_com << 16);
			}
		}
	}
}
#endif /* __USE_ENCODER */

/**
 *********************************************************************************************
 * @brief		Detect _motor position
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none
 *********************************************************************************************
 */
void D_Detect_Rotor_Position(vector_t* const _motor)
{

	q31_t temp32_r, temp32_Lq;
	q31_t temp32_OmegaErrI, temp32_OmegaErrP;
	q31_t omega_calc;

	/**
	 * @note	Voltage equation
	 * 			Vd = R*Id + Ld*pId - OMEGA-est*Lq + Ed
	 * 			  (p = d/dt, Id=Constant value -> pId = 0)
	 */
	if (_motor->drv.vector_cmd.F_vcomm_Edetect == SET)
	{
		/* Voltage drop of resistance	 V = R * I */
		temp32_r = (_motor->para.motor.r * (q31_t)_motor->drv.Id) << 1;


		/* Voltage drop of inductance	 V = 2PIfL * I */
		temp32_Lq = (((_motor->drv.omega.half[1] * _motor->para.motor.Lq) << 1) + cROUND_BIT15) >> 16;	/* Q15 * Q15 -> Q15 */
		temp32_Lq = (temp32_Lq * _motor->drv.Iq) << 1;													/* Q15 * Q15 -> Q31 */

		/* d-axis induced voltage */
		/* Ed = Vd - R*Id + OMEGA-est*Lq */
		_motor->drv.Ed = (q15_t)((_motor->drv.Vd.word - temp32_r + temp32_Lq + cROUND_BIT15) >> 16);

		/**
		 * @note	OMEGAest = OMEGAref - K*Ed
		 */
		/* Set integration value */
#if defined(__POSITION_KI_RANGE)
		/* OmegaErr(I) * Hz_MAX = FIX_12 * Position_Ki * CtrlPrd * Ed * V_MAX */
		temp32_OmegaErrI = (_motor->para.pos.ki * (q31_t)_motor->drv.Ed) << 4;
#else
		/* OmegaErr(I) * Hz_MAX = FIX_15 * Position_Ki * CtrlPrd * Ed * V_MAX */
		temp32_OmegaErrI = (_motor->para.pos.ki * (q31_t)_motor->drv.Ed) << 1;
#endif

		if (_motor->drv.omega_com.half[1] >= 0)
		{
			_motor->drv.Ed_I = __QSUB(_motor->drv.Ed_I, temp32_OmegaErrI);
		}
		else
		{
			_motor->drv.Ed_I = __QADD(_motor->drv.Ed_I, temp32_OmegaErrI);
		}

		/* Set proportionality value */
#if defined(__POSITION_KP_RANGE)
		/* OmegaErr(P) * Hz_MAX = FIX_12 * Position_Kp * Ed * V_MAX */
		temp32_OmegaErrP = (_motor->para.pos.kp * (q31_t)_motor->drv.Ed) << 4;
#else
		/* OmegaErr(P) * Hz_MAX = FIX_15 * Position_Kp * Ed * V_MAX */
		temp32_OmegaErrP = (_motor->para.pos.kp * (q31_t)_motor->drv.Ed) << 1;
#endif

		if (_motor->drv.omega_com.half[1] >= 0)
		{
			_motor->drv.Ed_PI = __QSUB(_motor->drv.Ed_I, temp32_OmegaErrP);
		}
		else
		{
			_motor->drv.Ed_PI = __QADD(_motor->drv.Ed_I, temp32_OmegaErrP);
		}

		/* Speed calculation value = Speed command value + Error PI value */
		omega_calc = __QADD(_motor->drv.omega_com.word, _motor->drv.Ed_PI);

		if (_motor->drv.vector_cmd.F_vcomm_omega == SET)
		{
			if (_motor->drv.command.encoder == CLEAR)
			{
				_motor->drv.omega.word = omega_calc;
			}
		}
		else
		{
			/* Initialize of proportionality value, integration value and Omega */
			_motor->drv.omega.word = _motor->drv.omega_com.word;
			_motor->drv.Ed_I = 0;
			_motor->drv.Ed_PI = 0;
		}

		/* Position estimation */
		if (_motor->drv.vector_cmd.F_vcomm_theta == SET)
		{
			if (_motor->drv.command.encoder == CLEAR)
			{
				/* estimated THETAn = THETAn-1 + OMEGA * t */
				_motor->drv.theta.word += ((_motor->drv.omega.half[1] * _motor->para.pos.ctrlprd) << 1);
			}
		}
		else
		{
			_motor->drv.theta.word = (q31_t)(_motor->drv.theta_com << 16);
		}
	}
	else
	{
		_motor->drv.theta.word = (q31_t)(_motor->drv.theta_com << 16);
		_motor->drv.Ed = 0;
	}
}

/**
 *********************************************************************************************
 * @brief		Speed control ( Caricurate q-axis current )
 *
 * @param		vector_t* const	_motor			The structure address for motor vector control.
 *
 * @return		none
 *********************************************************************************************
 */
void D_Control_Speed(vector_t* const _motor)
{
	/* Determination of R_Iq_ref */
	if (_motor->drv.vector_cmd.F_vcomm_current == SET)
	{
		q31_t temp32_PI;
		q31_t temp_iqref;

		/* Speed deviation */
		_motor->drv.omega_dev = (q15_t)(__QSUB(_motor->drv.omega_com.word, _motor->drv.omega.word) >> 16);

		/* Set proportionality value */
#if defined(__SPEED_KI_RANGE)
		temp32_PI = ((q31_t)_motor->para.spd.ki * _motor->drv.omega_dev) << 4;
#else
		temp32_PI = ((q31_t)_motor->para.spd.ki * _motor->drv.omega_dev) << 1;
#endif
		_motor->drv.Iq_ref_I.word = __QADD(_motor->drv.Iq_ref_I.word, temp32_PI);

		/* Limit value excess */
		if (_motor->drv.Iq_ref_I.word > _motor->para.iq_lim)
		{
			_motor->drv.Iq_ref_I.word = _motor->para.iq_lim;
		}
		else if (_motor->drv.Iq_ref_I.word < -_motor->para.iq_lim)
		{
			_motor->drv.Iq_ref_I.word = -_motor->para.iq_lim;
		}


		/* Set integration value */
#if defined(__SPEED_KP_RANGE)
		temp32_PI = (_motor->para.spd.kp * _motor->drv.omega_dev) << 4;
#else
		temp32_PI = (_motor->para.spd.kp * _motor->drv.omega_dev) << 1;
#endif
		temp_iqref = __QADD(_motor->drv.Iq_ref_I.word, temp32_PI);
		/* Limit value excess */
		if (temp_iqref > _motor->para.iq_lim)
		{
			_motor->drv.Iq_ref = (q15_t)(_motor->para.iq_lim >> 16);
		}
		else if (temp_iqref < -_motor->para.iq_lim)
		{
			_motor->drv.Iq_ref = (q15_t)(-_motor->para.iq_lim >> 16);
		}
		else
		{
			_motor->drv.Iq_ref = temp_iqref >> 16;
		}
	}
	else
	{
		/* Without Speed control. R_Iq_com -> R_Iq_ref */
		if (abs(_motor->drv.Iq_com.word) >= _motor->para.iq_lim)
		{
			/* Limit value excess */
			if (_motor->drv.Iq_com.word >= 0)
			{
				_motor->drv.Iq_ref = (q15_t)(_motor->para.iq_lim >> 16);
				_motor->drv.Iq_ref_I.word = _motor->para.iq_lim;
			}
			else
			{
				_motor->drv.Iq_ref = (q15_t)(-_motor->para.iq_lim >> 16);
				_motor->drv.Iq_ref_I.word = -_motor->para.iq_lim;
			}
		}
		else
		{
			/* Initialize of proportionality value and integration value */
			_motor->drv.Iq_ref = _motor->drv.Iq_com.half[1];
			_motor->drv.Iq_ref_I.word = _motor->drv.Iq_com.word;
		}

	}

	/* Determination of R_Id_ref */
	if (abs(_motor->drv.Id_com.word) >= _motor->para.id_lim)
	{
		/* Limit value excess */
		if (_motor->drv.Id_com.word >= 0)
		{
			_motor->drv.Id_ref = (q15_t)(_motor->para.id_lim >> 16);
		}
		else
		{
			_motor->drv.Id_ref = (q15_t)(-_motor->para.id_lim >> 16);
		}
	}
	else
	{
		_motor->drv.Id_ref = _motor->drv.Id_com.half[1];
	}

}

/**
 ********************************************************************************************
 * @brief		D_SetZeroCurrentByVE.
 *
 * @param[in]		vector_t* const	_motor	.
 * @param[in,out]	TSB_VE_TypeDef*	pVE		.
 *
 * @return		none
 ********************************************************************************************
 */
void	D_SetZeroCurrentByVE(vector_t* const _motor, TSB_VE_TypeDef* pVE)
{
	if (_motor->stage.main == cStop)
	{
		if (_motor->shunt_type == c3shunt)
		{
			uint32_t	adc_ia, adc_ib, adc_ic;
			VE_GetCurrentAdcData(pVE, &adc_ia, &adc_ib, &adc_ic);
			_motor->drv.Iao_ave.word += (((int32_t)(adc_ia << 16) - _motor->drv.Iao_ave.word) >> cIXO_AVE);
			_motor->drv.Ibo_ave.word += (((int32_t)(adc_ib << 16) - _motor->drv.Ibo_ave.word) >> cIXO_AVE);
			_motor->drv.Ico_ave.word += (((int32_t)(adc_ic << 16) - _motor->drv.Ico_ave.word) >> cIXO_AVE);
			VE_SetZeroCurrentData(pVE, _motor->drv.Iao_ave.half[1], _motor->drv.Ibo_ave.half[1], _motor->drv.Ico_ave.half[1]);

		}
		else if (_motor->shunt_type == c1shunt)
		{
			ADC_Result	i_adc = {0};
			uint32_t	adc_ia;

			i_adc = ADC_GetConvertResult(TSB_ADA, ADC_REG0);
			i_adc.All &= 0xfff0;

			_motor->drv.Iao_ave.word += (((int32_t)(i_adc.All << 16) - _motor->drv.Iao_ave.word) >> cIXO_AVE);

			adc_ia = _motor->drv.Iao_ave.half[1];
			VE_SetZeroCurrentData(pVE, adc_ia, adc_ia, adc_ia);
		}
	}
}



/**
 *********************************************************************************************
 * @brief		Check over current.
 *
 * @param		q15_t			_ovc	current of over error value.
 * @param		q15_t			_iu		current of u phase.
 * @param		q15_t			_iv		current of v phase.
 * @param		q15_t			_iw		current of w phase.
 *
 * @return		bool	TRUE:Over current / FALSE:Nomal
 *********************************************************************************************
 */
bool D_Check_OverCurrent(q15_t _ovc, q15_t _iu, q15_t _iv, q15_t _iw)
{
	if ((abs(_iu) > _ovc)
		|| (abs(_iv) > _ovc)
		|| (abs(_iw) > _ovc))
	{
		return (true);
	}
	else
	{
		return (false);
	}
}



/**
 *********************************************************************************************
 * @brief		D_CalSector
 *
 * @param		q31_t 		_valpha		alpha-axis Valpha
 * @param		q31_t 		_vbeta		beta-axis Vbeta
 *
 * @return		sector	
 *********************************************************************************************
 */
uint8_t D_CalSector(q31_t _valpha, q31_t _vbeta)
{
	#define	cCAL_1_DIV_SQRT3	(int32_t)(FIX_15 * 1 / 1.73205f)	/* Q15 1/sqrt3 */

  	uint8_t sector = 0;
	q31_t temp_vbeta = ((((int64_t)abs(_vbeta)) * cCAL_1_DIV_SQRT3) >> 15);

	if ((_valpha >= 0) && (_vbeta >= 0))
	{
		sector = (_valpha >= temp_vbeta)? 0: 1;
	}
	else if ((_valpha < 0) && (_vbeta >= 0))
	{
		sector = (abs(_valpha) < temp_vbeta)? 1: 2;
	}
	else if ((_valpha < 0) && (_vbeta < 0))
	{
		sector = (abs(_valpha) < temp_vbeta)? 4: 3;
	}
	else if ((_valpha >= 0) && (_vbeta < 0))
	{
		sector = (_valpha < temp_vbeta)? 4: 5;
	}
	return(sector);
}

/**
 *********************************************************************************************
 * @brief		D_SVM
 *
 * @param		q31_t 		_valpha		alpha-axis Valpha
 * @param		q31_t 		_vbeta		beta-axis Vbeta
 * @param		q15_t	 	_vdc		DC voltage
 * @param		uint8_t 	_sector		sector
 * @param		uint8_t 	_modul		modul
 * @param		uint16_t* 	_p_vu		U phase duty ratio storage address
 * @param		uint16_t* 	_p_vv		V phase duty ratio storage address
 * @param		uint16_t* 	_p_vw		W phase duty ratio storage address
 *
 * @return		none	
 *********************************************************************************************
 */
void D_SVM(q31_t _valpha, q31_t _vbeta, q15_t _vdc, uint8_t _sector, uint8_t _modul, uint16_t* _p_vu, uint16_t* _p_vv, uint16_t* _p_vw)
{
	#define cSVM_K				(int64_t)((1L<<15) * 3.0f / 2.0f)			/* Q15 3/2 */
	#define	cSVM_1_DIV_SQRT3	(int64_t)((1L<<15) * 1.0f / 1.7320508f)		/* Q15 1/sqrt3 */
	#define	cSVM_2_DIV_SQRT3	(int64_t)((1L<<15) * 2.0f / 1.7320508f)		/* Q15 2/sqrt3 */

	#define cSVM_100_PERCENT	(int32_t)(((1LL<<31) * 1) - 1) /* Q31 1.0 */

	int32_t temp_1, temp_2;
	q31_t d0, d1, d2;

	switch (_sector)
	{
		case 0:
			/* Sector0 */
			/* D1 = k * (Valpha - 1/sqrt3 * Vbeta) / Vdc */
			/* D2 = k * ( 2/sqrt3 * Vbeta) / Vdc */
			/* D0 = 100% - D1 - D2 */
			temp_1 = (int32_t)(__QSUB(_valpha, ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15)));
			temp_2 = (int32_t)((cSVM_2_DIV_SQRT3 * _vbeta) >> 15);
			d1 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
			d2 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
			d0 = (q31_t)__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2);
			/* Set Duty */
			if (_modul == 0)
			{
				/* 3phase modulation */
				/* U = D1 + D2 + (D0 / 2) */
				/* V = D2 + (D0 / 2) */
				/* W = D0 / 2 */
				*_p_vu = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
				*_p_vv = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
				*_p_vw = (uint16_t)((d0 / 2) >> 16);
			}
			else
			{
				/* 2phase modulation */
				/* U = D1 + D2 */
				/* V = D2 */
				/* W = 0 */
				*_p_vu = (uint16_t)(__QADD(d1, d2) >> 16);
				*_p_vv = (uint16_t)(d2 >> 16);
				*_p_vw = 0;
			}
			break;

		case 1:
			/* Sector1 */
			/* D1 = k * (-Valpha + 1/sqrt3 * Vbeta) / Vdc */
			/* D2 = k * (Valpha + 1/sqrt3 * Vbeta) / Vdc */
			/* D0 = 100% - D1 - D2 */
			temp_1 = (int32_t)__QADD((-_valpha), ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			temp_2 = (int32_t)__QADD(_valpha, ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			d1 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
			d2 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
			d0 = (q31_t)__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2);
			/* Set Duty */
			if (_modul == 0)
			{
				/* 3phase modulation */
				/* U = D2 + (D0 / 2) */
				/* V = D1 + D2 + (D0 / 2) */
				/* W = D0 / 2 */
				*_p_vu = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
				*_p_vv = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
				*_p_vw = (uint16_t)((d0 / 2) >> 16);
			}
			else
			{
				/* 2phase modulation */
				/* U = D2 */
				/* V = D1 + D2 */
				/* W = 0 */
				*_p_vu = (uint16_t)(d2 >> 16);
				*_p_vv = (uint16_t)(__QADD(d1, d2) >> 16);
				*_p_vw = 0;
			}
			break;

		case 2:
			/* Sector2 */
			/* D1 = k * ( 2/sqrt3 * Vbeta) / Vdc */
			/* D2 = k * (-Valpha - 1/sqrt3 * Vbeta) / Vdc */
			/* D0 = 100% - D1 - D2 */
			temp_1 = (int32_t)((cSVM_2_DIV_SQRT3 * _vbeta) >> 15);
			temp_2 = (int32_t)__QSUB((-_valpha), ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			d1 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
			d2 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
			d0 = (q31_t)__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2);
			/* Set Duty */
			if (_modul == 0)
			{
				/* 3phase modulation */
				/* U = D0 / 2 */
				/* V = D1 + D2 + (D0 / 2) */
				/* W = D2 + (D0 / 2) */
				*_p_vu = (uint16_t)((d0 / 2) >> 16);
				*_p_vv = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
				*_p_vw = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
			}
			else
			{
				/* 2phase modulation */
				/* U = 0 */
				/* V = D1 + D2 */
				/* W = D2 */
				*_p_vu = 0;
				*_p_vv = (uint16_t)(__QADD(d1, d2) >> 16);
				*_p_vw = (uint16_t)(d2 >> 16);
			}
			break;
		
		case 3:
			/* Sector3 */
			/* D1 = k * (-2/sqrt3 * Vbeta) / Vdc */
			/* D2 = k * (-Valpha + 1/sqrt3 * Vbeta) / Vdc */
			/* D0 = 100% - D1 - D2 */
			temp_1 = (int32_t)((cSVM_2_DIV_SQRT3 * _vbeta) >> 15);
			temp_2 = (int32_t)__QADD((-_valpha), ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			d1 = (q31_t)Limit_sub((int64_t)(cSVM_K * (-temp_1 /_vdc)), 0, cSVM_100_PERCENT);
			d2 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
			d0 = (q31_t)__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2);
			/* Set Duty */
			if (_modul == 0)
			{
				/* 3phase modulation */
				/* U = D0 / 2*/
				/* V = D2 + (D0 / 2) */
				/* W = D1 + D2 + (D0 / 2) */
				*_p_vu = (uint16_t)((d0 / 2) >> 16);
				*_p_vv = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
				*_p_vw = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
			}
			else
			{
				/* 2phase modulation */
				/* U = 0 */
				/* V = D2 */
				/* W = D1 + D2 */
				*_p_vu = 0;
				*_p_vv = (uint16_t)(d2 >> 16);
				*_p_vw = (uint16_t)(__QADD(d1, d2) >> 16);
			}
			break;

		case 4:
			/* Sector4 */
			/* D1 = k * (-Valpha - 1/sqrt3 * Vbeta) / Vdc */
			/* D2 = k * (Valpha - 1/sqrt3 * Vbeta) / Vdc */
			/* D0 = 100% - D1 - D2 */
			temp_1 = (int32_t)__QSUB((-_valpha), ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			temp_2 = (int32_t)__QSUB(_valpha, ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			d1 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
			d2 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_2 / _vdc)), 0, cSVM_100_PERCENT);
			d0 = (q31_t)__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2);
			/* Set Duty */
			if (_modul == 0)
			{
				/* 3phase modulation */
				/* U = D2 + (D0 / 2) */
				/* V = D0 / 2 */
				/* W = D1 + D2 + (D0 / 2) */
				*_p_vu = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
				*_p_vv = (uint16_t)((d0 / 2) >> 16);
				*_p_vw = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
			}
			else
			{
				/* 2phase modulation */
				/* U = D2 */
				/* V = 0 */
				/* W = D1 + D2 */
				*_p_vu = (uint16_t)(d2 >> 16);
				*_p_vv = 0;
				*_p_vw = (uint16_t)(__QADD(d1, d2) >> 16);
			}
			break;

		case 5:
			/* Sector5 */
			/* D1 = k * (Valpha + 1/sqrt3 * Vbeta) / Vdc */
			/* D2 = k * ( -2/sqrt3 Vbeta) / Vdc */
			/* D0 = 100% - D1 - D2 */
			temp_1 = (int32_t)__QADD(_valpha, ((cSVM_1_DIV_SQRT3 * _vbeta) >> 15));
			temp_2 = (int32_t)((cSVM_2_DIV_SQRT3 * _vbeta) >> 15);
			d1 = (q31_t)Limit_sub((int64_t)(cSVM_K * (temp_1 / _vdc)), 0, cSVM_100_PERCENT);
			d2 = (q31_t)Limit_sub((int64_t)(cSVM_K * (-temp_2 / _vdc)), 0, cSVM_100_PERCENT);
			d0 = (q31_t)__QSUB(__QSUB(cSVM_100_PERCENT, d1), d2);
			/* Set Duty */
			if (_modul == 0)
			{
				/* 3phase modulation */
				/* U = D1 + D2 + (D0 / 2) */
				/* V = D0 / 2 */
				/* W = D2 + (D0 / 2) */
				*_p_vu = (uint16_t)(__QADD(__QADD((d0 / 2), d1), d2) >> 16);
				*_p_vv = (uint16_t)((d0 / 2) >> 16);
				*_p_vw = (uint16_t)(__QADD((d0 / 2), d2) >> 16);
			}
			else
			{
				/* 2phase modulation */
				/* U = D1 + D2 */
				/* V = 0 */
				/* W = D2 */
				*_p_vu = (uint16_t)(__QADD(d1, d2) >> 16);
				*_p_vv = 0;
				*_p_vw = (uint16_t)(d2 >> 16);
			}
			break;
		default:
			break;
	}
}



/**
 *********************************************************************************************
 * @brief		Limitation.
 *
 * @param		int64_t			_indata			Q31 input data(Double precision).
 * @param		int32_t			_limlo			Q31 lower limit value.
 * @param		int32_t			_limup			Q31 upper limit value.
 *
 * @return		int32_t			Q31
 *********************************************************************************************
 */
static int32_t Limit_sub(int64_t _indata, int32_t _limlo, int32_t _limup)
{
  	int32_t ret = (int32_t)_indata;
 
	if (_indata > (int64_t)_limup)
	{
		ret = _limup;
	}
	if (_indata < (int64_t)_limlo)
	{
		ret = _limlo;
	}
	return (ret);
}

/*********************************** END OF FILE ******************************/
