/**
 *******************************************************************************
 * @file    ipdrv_adc.c
 * @brief   This file provides API functions for M4Kx ADC driver.
 * @version V1.0
 *
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "ipdrv_adc.h"

/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup ADC
  * @brief ADC driver modules
  * @{
  */

/** @defgroup ADC_Private_Defines
  * @{
  */
#define ADC_MASK_KEEP1BIT       ((uint32_t)0x00000001)

#define ADC_MASK_SINGLE         ((uint32_t)0x00000081)
#define ADC_MASK_CONTINUE       ((uint32_t)0x00000082)

/** @} */
/* End of group ADC_Private_Defines */

/** @defgroup ADC_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group ADC_Private_FunctionPrototypes */

/** @defgroup ADC_Private_Functions
  * @{
  */

static const volatile uint32_t *const ADA_REGx_Address[32U] = {
    &TSB_ADA->REG0, &TSB_ADA->REG1, &TSB_ADA->REG2, &TSB_ADA->REG3,
    &TSB_ADA->REG4, &TSB_ADA->REG5, &TSB_ADA->REG6, &TSB_ADA->REG7,
    &TSB_ADA->REG8, &TSB_ADA->REG9, &TSB_ADA->REG10, &TSB_ADA->REG11,
    &TSB_ADA->REG12, &TSB_ADA->REG13, &TSB_ADA->REG14, &TSB_ADA->REG15,
    &TSB_ADA->REG16, &TSB_ADA->REG17, &TSB_ADA->REG18, &TSB_ADA->REG19,
    &TSB_ADA->REG20, &TSB_ADA->REG21, &TSB_ADA->REG22, &TSB_ADA->REG23,
    &TSB_ADA->REG24, &TSB_ADA->REG25, &TSB_ADA->REG26, &TSB_ADA->REG27,
    &TSB_ADA->REG28, &TSB_ADA->REG29, &TSB_ADA->REG30, &TSB_ADA->REG31
};

static volatile uint32_t *const ADA_TSETx_Address[32U] = {
    &TSB_ADA->TSET0, &TSB_ADA->TSET1, &TSB_ADA->TSET2, &TSB_ADA->TSET3,
    &TSB_ADA->TSET4, &TSB_ADA->TSET5, &TSB_ADA->TSET6, &TSB_ADA->TSET7,
    &TSB_ADA->TSET8, &TSB_ADA->TSET9, &TSB_ADA->TSET10, &TSB_ADA->TSET11,
    &TSB_ADA->TSET12, &TSB_ADA->TSET13, &TSB_ADA->TSET14, &TSB_ADA->TSET15,
    &TSB_ADA->TSET16, &TSB_ADA->TSET17, &TSB_ADA->TSET18, &TSB_ADA->TSET19,
    &TSB_ADA->TSET20, &TSB_ADA->TSET21, &TSB_ADA->TSET22, &TSB_ADA->TSET23,
    &TSB_ADA->TSET24, &TSB_ADA->TSET25, &TSB_ADA->TSET26, &TSB_ADA->TSET27,
    &TSB_ADA->TSET28, &TSB_ADA->TSET29, &TSB_ADA->TSET30, &TSB_ADA->TSET31
};

/** @} */
/* End of group ADC_Private_Functions */


/** @defgroup ADC_Exported_Functions
  * @{
  */

/**
  * @brief  Set ADC prescaler output(SCLK) of the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_ADA.
  * @param  Sampling_Period: Select ADC sample period time.
  *   This parameter can be one of the following values:
  *   ADC_SAMPLING_PERIOD_1, ADC_SAMPLING_PERIOD_2, ADC_SAMPLING_PERIOD_3,
  *   ADC_SAMPLING_PERIOD_4, ADC_SAMPLING_PERIOD_16, ADC_SAMPLING_PERIOD_64,
  *   ADC_SAMPLING_PERIOD_128, ADC_SAMPLING_PERIOD_256, ADC_SAMPLING_PERIOD_512,
  *   ADC_SAMPLING_PERIOD_1024.
  * @param  Prescaler_Output: Select ADC prescaler output.
  *   This parameter can be one of the following values:
  *   ADC_SCLK_1,   ADC_SCLK_2, 
  *   ADC_SCLK_4,   ADC_SCLK_8,
  *   ADC_SCLK_16
  * @retval None
  * @register The used registers:
  *   ADCCLK<EXAZ[6:3]><VADCLK[2:0]>
  */
void ADC_SetClk(TSB_AD_TypeDef * ADx, uint32_t Sampling_Period, uint32_t Prescaler_Output)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_SAMPLING_PERIOD(Sampling_Period));
    assert_param(IS_ADC_PRESCALER(Prescaler_Output));

    tmp = Sampling_Period | Prescaler_Output;
    ADx->CLK = tmp;
}


/**
  * @brief  Enable the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_ADA.
  * @param  ClkMod: Select clock mode.
  *   This parameter can be one of the following values:
  *   Conversion time. 0.5us:ADC_CLKMOD_0_5, 0.85us:ADC_CLKMOD_0_85
  * @param  Sel: Select VREFHx/AINn switching control.
  *   This parameter can be one of the following values:
  *   VREFHx:ADC_REFSEL_VREFH, AINn:ADC_REFSEL_AIN
  * @param  Rcut: Select Low power consumption mode.
  *   This parameter can be one of the following values:
  *   Normal:ADC_RCUT_NORMAL, Low power:ADC_RCUT_LOWPOWER
  * @retval None
  * @register The used registers:
  *   ADCMOD0<REFBSEL><RCUT><DACON>
  *   ADCCR0<ADEN>
  */
void ADC_Enable(TSB_AD_TypeDef * ADx, ADC_ClkMod ClkMod, ADC_RefSel Sel, ADC_RcutMode Rcut)
{
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_CLKMOD(ClkMod));
    assert_param(IS_ADC_REFSEL(Sel));
    assert_param(IS_ADC_RCUTMODE(Rcut));

    ADx->MOD0 = (ClkMod << 3U) | (Sel << 2U) | (Rcut << 1U) | (1U << 0U);
    if (ClkMod ==ADC_CLKMOD_0_5) {
        ADx->MOD1 = 0x00304000;    
    } else {
        ADx->MOD1 = 0x00304111;    
    }
    TSB_ADA_CR0_ADEN = 1U;

}

/**
  * @brief  Start the specified ADC unit with software trigger or constant trigger.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_ADA.
  * @param  Trg: Select trigger type.
  *   This parameter can be one of the following values:
  *   ADC_TRG_SINGLE, ADC_TRG_CONTINUE
  * @retval None
  * @register The used registers:
  *   ADCCR0<SGL><CNT>
  */
void ADC_Start(TSB_AD_TypeDef * ADx, ADC_TrgType Trg)
{
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_START(Trg));

    if (Trg == ADC_TRG_SINGLE) {
        tmp = ADx->CR0 & ADC_MASK_SINGLE;
        tmp |= (uint32_t) (0x01U << 1U);
        ADx->CR0 = tmp;
    } else {                    /* ADC_TRG_CONSTANT */
        tmp = ADx->CR0 & ADC_MASK_CONTINUE;
        tmp |= (uint32_t) (0x01U << 0U);
        ADx->CR0 = tmp;
    }
}


/**
  * @brief  Get the conversion state of the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *   TSB_ADA.
  * @param  Trg: Select trigger type.
  *   This parameter can be one of the following values:
  *   ADC_TRG_SINGLE, ADC_TRG_CONTINUE
  * @retval ADC conversion state.
  *   The value returned can be one of the followings:
  *   DONE, BUSY.
  * @register The used register:
  *   ADCST
  */
WorkState ADC_GetConvertState(TSB_AD_TypeDef * ADx, ADC_TrgType Trg)
{
    WorkState ret = DONE;
    uint32_t tmp = 0U;

    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_TRGTYPE(Trg));

    tmp = ADx->ST;
    if (Trg == ADC_TRG_SINGLE) {
        tmp &= (ADC_MASK_KEEP1BIT << 2U);       /* only keep bit2 */
    } else {
        tmp &= (ADC_MASK_KEEP1BIT << 3U);       /* only keep bit3 */
    }

    if (tmp) {
        ret = BUSY;
    } else {
        /* Do nothing */
    }

    return ret;
}

/**
  * @brief  Get result from the specified AD Conversion Result Register.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *     TSB_ADA.
  * @param  ResultREGx: Select which ADC result register will be read.
  *   This parameter can be one of the following values:
  *     ADC_REG0, ADC_REG1, ADC_REG2, ADC_REG3, ADC_REG4, ADC_REG5,
  *     ADC_REG6, ADC_REG7, ADC_REG8, ADC_REG9, ADC_REG10, ADC_REG11,
  *     ADC_REG12, ADC_REG13, ADC_REG14, ADC_REG15, ADC_REG16, ADC_REG17,
  *     ADC_REG18, ADC_REG19, ADC_REG20, ADC_REG21, ADC_REG22, ADC_REG23,
  *     ADC_REG24, ADC_REG25, ADC_REG26, ADC_REG27, ADC_REG28, ADC_REG29,
  *     ADC_REG30, ADC_REG31
  * @retval A union with AD result and 2 bits extra information.
  *         Refer to the members of ADC_Result in .h file for more detail usage.
  * @register The used registers:
  *   ADCREGn (n can be 0 to 31)   
  */
ADC_Result ADC_GetConvertResult(TSB_AD_TypeDef * ADx, ADC_REGx ResultREGx)
{
    ADC_Result result = { 0U };
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_REGx(ResultREGx));

    /* read out result register wanted */
    /* only keep bit15 to bit0 */
    if (ADx == TSB_ADA) {
        result.All = (*(ADA_REGx_Address[ResultREGx])) & (uint32_t) 0xFFFFU;
    } else {
        result.All = 0;
    }

    return result;
}


/**
  * @brief  Set Software Trigger Program Register of the specified ADC unit.
  * @param  ADx: Select ADC unit
  *   This parameter can be one of the following values:
  *     TSB_ADA
  * @param  ResultREGx: Select which ADC result register will be used for the specified ADC unit.
  *   This parameter can be one of the following values:
  *     ADC_REG0, ADC_REG1, ADC_REG2, ADC_REG3, ADC_REG4, ADC_REG5,
  *     ADC_REG6, ADC_REG7, ADC_REG8, ADC_REG9, ADC_REG10, ADC_REG11,
  *     ADC_REG12, ADC_REG13, ADC_REG14, ADC_REG15, ADC_REG16, ADC_REG17,
  *     ADC_REG18, ADC_REG19, ADC_REG20, ADC_REG21, ADC_REG22, ADC_REG23,
  *     ADC_REG24, ADC_REG25, ADC_REG26, ADC_REG27, ADC_REG28, ADC_REG29,
  *     ADC_REG30, ADC_REG31
  * @param  MacroAINx: Select AD Channel together with its enabled or disabled setting.
  *   This parameter must be inputed with macro as the format below:
  *     TRG_ENABLE(x), TRG_DISABLE(x).
  *          ADC_AIN0, ADC_AIN1, ADC_AIN2, ADC_AIN3, ADC_AIN4,
  *          ADC_AIN5, ADC_AIN6, ADC_AIN7, ADC_AIN8, ADC_AIN9,
  *          ADC_AIN10, ADC_AIN11, ADC_AIN12, ADC_AIN13, ADC_AIN14,
  *          ADC_AIN15, ADC_AIN16, ADC_AIN17, ADC_AIN18, ADC_AIN19, 
  *          ADC_AIN20, ADC_AIN21, ADC_AIN22, ADC_AIN23 
  * @retval None
  * @register The used registers:
  *   ADCTSETn (n can be 0 to 31) 
  */
void ADC_SetSWTrg(TSB_AD_TypeDef * ADx, ADC_REGx ResultREGx, uint8_t MacroAINx)
{
    /* Check the parameters */
    assert_param(IS_ADC_UNIT(ADx));
    assert_param(IS_ADC_REGx(ResultREGx));
    if (ADx == TSB_ADA) {
        assert_param(IS_UNIT_A_AINx(MacroAINx));
    } else {
        assert_param(IS_UNIT_B_AINx(MacroAINx));
    }

	*(ADA_TSETx_Address[ResultREGx]) = ((GEN_TRG_SINGLE << 5) | MacroAINx);

}


/** @} */
/* End of group ADC_Exported_Functions */

/** @} */
/* End of group ADC */

/** @} */
/* End of group TX04_Periph_Driver */

