/**
 *******************************************************************************
 * @file    ipdrv_cg.c
 * @brief   This file provides API functions for M4Kx CG driver
 * @version V1.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *******************************************************************************
 */

/* Includes ------------------------------------------------------------------*/
#include "ipdrv_cg.h"

/** @addtogroup TX04_Periph_Driver
  * @{
  */

/** @defgroup CG
  * @brief CG driver modules
  * @{
  */

/** @defgroup CG_Private_Defines
  * @{
  */

#define CG_FSYSENB_WRITE1_MASK               ((uint32_t)0x70000000)
#define CG_FCEN_WRITE0_MASK                  ((uint32_t)0x00000080)
#define ADC_ST_ADBF_MASK                     ((uint32_t)0x00000080)

/** @} */
/* End of group CG_Private_Defines */

/** @defgroup CG_Private_FunctionPrototypes
  * @{
  */

/** @} */
/* End of group CG_Private_FunctionPrototypes */

/** @defgroup CG_Private_Functions
  * @{
  */

/** @} */
/* End of group CG_Private_Functions */

/** @defgroup CG_Exported_Functions
  * @{
  */


 /**
   * @brief  Enable or disable supplying clock fsys to peripheries
   * @param  Periph: The target peripheral of CG supplies clock
   *   This parameter can be one of the following values or their combination:
   * CG_FC_PERIPH_PORTA, CG_FC_PERIPH_PORTB, CG_FC_PERIPH_PORTC
   * , CG_FC_PERIPH_PORTD, CG_FC_PERIPH_PORTE, CG_FC_PERIPH_PORTF
   * , CG_FC_PERIPH_PORTG, CG_FC_PERIPH_PORTH, CG_FC_PERIPH_PORTJ
   * , CG_FC_PERIPH_PORTK, CG_FC_PERIPH_PORTL, CG_FC_PERIPH_TSPI0
   * , CG_FC_PERIPH_TSPI1, CG_FC_PERIPH_TSPI2, CG_FC_PERIPH_TSPI3
   * , CG_FC_PERIPH_UART0, CG_FC_PERIPH_UART1, CG_FC_PERIPH_UART2
   * , CG_FC_PERIPH_UART3, CG_FC_PERIPH_I2C, CG_FC_PERIPH_T32A0
   * , CG_FC_PERIPH_T32A1, CG_FC_PERIPH_T32A2, CG_FC_PERIPH_T32A3
   * , CG_FC_PERIPH_T32A4, CG_FC_PERIPH_T32A5, CG_FC_PERIPH_RTC
   * @param  NewState: New state of clock supply setting.
   *   This parameter can be one of the following values:
   *   DISABLE or ENABLE
   * @retval None
   */
void CG_SetFsysSupplyPeriphA(uint32_t Periph, FunctionalState NewState)
{
    assert_param(IS_FUNCTIONAL_STATE(NewState));
    assert_param(IS_CG_FC_PERIPHA(Periph));

    if (NewState == ENABLE) {   /* write '1' to enable */
        TSB_CG->FSYSENA |= Periph;
    } else {                    /* clear to '0' to disable */
        TSB_CG->FSYSENA &= ~Periph;
    }
}

 /**
   * @brief  Enable or disable supplying clock fsys to peripheries
   * @param  Periph: The target peripheral of CG supplies clock
   *   This parameter can be one of the following values or their combination:   
   * CG_FC_PERIPH_ADC0, CG_FC_PERIPH_ADC1, CG_FC_PERIPH_AMP
   * , CG_FC_PERIPH_CMP, CG_FC_PERIPH_DAC0, CG_FC_PERIPH_DAC1
   * , CG_FC_PERIPH_DAC2, CG_FC_PERIPH_A_ENC, CG_FC_PERIPH_PMD0
   * , CG_FC_PERIPH_PMD1, CG_FC_PERIPH_A_VE_P, CG_FC_PERIPH_RAM_PARI
   * , CG_FC_PERIPH_CRC, CG_FC_PERIPH_OFD, CG_FC_PERIPH_TRIM
   * , CG_FC_PERIPH_TRGSEL, CG_FC_PERIPH_DMAC, CG_FC_PERIPH_SIWDT
   * @param  NewState: New state of clock supply setting.
   *   This parameter can be one of the following values:
   *   DISABLE or ENABLE
   * @retval None
   */
void CG_SetFsysSupplyPeriphB(uint32_t Periph, FunctionalState NewState)
{
    uint32_t regval = TSB_CG->FSYSENB;

    assert_param(IS_FUNCTIONAL_STATE(NewState));
    assert_param(IS_CG_FC_PERIPHB(Periph));

    if (NewState == ENABLE) {   /* write '1' to enable */
        regval |= Periph;
    } else {                    /* clear to '0' to disable */
        regval &= ~Periph;
    }

    regval |= CG_FSYSENB_WRITE1_MASK;  /* CGFSYSENB bit 30-28 write "1" */
    TSB_CG->FSYSENB = regval;
}


 /**
   * @brief  Enable or disable supplying clock fc to peripheries
   * @param  Periph: The target peripheral of CG supplies clock
   *   This parameter can be one of the following values or their combination:   
   *   CG_FC_PERIPH_DNFCK, CG_FC_PERIPH_ALL.
   * @param  NewState: New state of clock supply setting.
   *   This parameter can be one of the following values:
   *   DISABLE or ENABLE
   * @retval None
   */
void CG_SetFcSupplyPeriph(uint32_t Periph, FunctionalState NewState)
{
    uint32_t regval = TSB_CG->FCEN;

    assert_param(IS_FUNCTIONAL_STATE(NewState));
    assert_param(IS_CG_FC_PERIPH(Periph));

    if (NewState == ENABLE) {   /* write '1' to enable */
        regval |= Periph;
    } else {                    /* clear to '0' to disable */
        regval &= ~Periph;
    }

    regval &= CG_FCEN_WRITE0_MASK;  /* CGFCEN bit 10-31 write "0" */
    TSB_CG->FCEN = regval;
}


/**
  * @brief  Enable or disable supplying clock (fc or fplladc) for ADC.
  * @param  NewState: New state of clock (fc or fplladc) supply setting for ADC.
  *   This parameter can be one of the following values:
  *   ENABLE or DISABLE.
  * @retval None.
  */
void CG_SetAdcClkSupply(FunctionalState NewState)
{
    /* Check the parameters */
    assert_param(IS_FUNCTIONAL_STATE(NewState));

    /* Check ADC is not during conversion */
    while ((TSB_ADA->ST & ADC_ST_ADBF_MASK) != 0U) {
        /* Do nothing */
    }

    /* Set CGSPCLKEN<ADCKEN>  */
    if (NewState == ENABLE) {
        TSB_CG_SPCLKEN_ADCKEN = 1U;
    } else {
        TSB_CG_SPCLKEN_ADCKEN = 0U;
    }
}



/** @} */
/* End of group CG_Exported_Functions */

/** @} */
/* End of group CG */

/** @} */
/* End of group TX04_Periph_Driver */

