/**
 ****************************************************************************
 * @file	dac_drv.c
 * @brief	DAC IC Control for AD5724 on TSPI.
 * @version	V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */
#include "ipdefine.h"

#include "ipdrv_tspi.h"
#include "ipdrv_cg.h"

#define DEFINE_APP
#include "dac_drv.h"
#undef	DEFINE_APP

/**
 * @note	DAC IC Control for AD5724.
 *
 */

/*===================================================================*
  Device local functions prototype.
 *===================================================================*/
static void DacLocalWriteInit(TSB_TSPI_TypeDef* const TSPIx);

/*===================================================================*
	  Local Variable Definition
 *===================================================================*/
uint32_t DacNo = DAC_MAX_CH;


/*===================================================================*
	  Const Definition
 *===================================================================*/


/* Define AD5624 Initial Command */

#define	cDAC_POWERCON	BIT32(0,0,0,0,0,0,0,0, 0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1)
/*                                             | | | | | | | | | | | | | | | | | | | | | | | +----; D0
                                               | | | | | | | | | | | | | | | | | | | | | | +------; D1
                                               | | | | | | | | | | | | | | | | | | | | | +--------; D2
                                               | | | | | | | | | | | | | | | | | | | | +----------; D3
                                               | | | | | | | | | | | | | | | | | | | +------------; D4
                                               | | | | | | | | | | | | | | | | | | +--------------; D5
                                               | | | | | | | | | | | | | | | | | +----------------; D6
                                               | | | | | | | | | | | | | | | | +------------------; D7
                                               | | | | | | | | | | | | | | | +--------------------; D8
                                               | | | | | | | | | | | | | | +----------------------; D9
                                               | | | | | | | | | | | | | +------------------------; D10
                                               | | | | | | | | | | | | +--------------------------; D11
                                               | | | | | | | | | | | +----------------------------; D12
                                               | | | | | | | | | | +------------------------------; D13
                                               | | | | | | | | | +--------------------------------; D14
                                               | | | | | | | | +----------------------------------; D15
                                               | | | | | | | +------------------------------------; A0
                                               | | | | | | +--------------------------------------; A1
                                               | | | | | +----------------------------------------; A2
                                               | | | | +------------------------------------------; C0
                                               | | | +--------------------------------------------; C1
                                               | | +----------------------------------------------; C2
                                               | +------------------------------------------------; x
                                               +--------------------------------------------------; x
*/


/**
 *********************************************************************************************
 * @brief		D/A initialize.
 *
 * @param		TSB_TSPI_TypeDef * const	TSPIx		Select the tspi channel.
 *
 * @return		none
 *********************************************************************************************
 */
void init_Dac(TSB_TSPI_TypeDef* const TSPIx)
{
	/* Single transfer */
	uint8_t transfer_type = 0U;			/* 0: Single transfer   1: Burst transfer 1Byte  */

	if (TSPIx == TSB_TSPI3)
	{
		CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_TSPI3, ENABLE);
	}
	else if (TSPIx == TSB_TSPI2)
	{
		CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_TSPI2, ENABLE);
	}
	else if (TSPIx == TSB_TSPI1)
	{
		CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_TSPI1, ENABLE);
	}
	else if (TSPIx == TSB_TSPI0)
	{
		CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_TSPI0, ENABLE);
	}

	/* maximum clock: clk = phiT0/2, divider=1 */
	TSPI_BaudClock clk = TSPI_BR_CLOCK_1;
	TSPI_Divider divider = TSPI_BR_DIVIDER_1;	/* CLK 20MHz@80MHz */

	TSPI_InitTypeDef init = { 0U };

	TSPI_Enable(TSPIx);
	TSPI_SWReset(TSPIx);

	TSPI_SetTransferNum(TSPIx, transfer_type);
	TSPI_SetTxRxCtrl(TSPIx, ENABLE);
	TSPI_SelectMode(TSPIx, TSPI_MODE_SIO);
	TSPI_SelectTransferMode(TSPIx, TSPI_TRMODE_TX);

	TSPI_SetFIFOLevelINT(TSPIx, TSPI_TX, 0U);
	TSPI_SetFIFOLevelINT(TSPIx, TSPI_RX, 1U);

	/* enable TSPI interrupt */
	TSPI_SetINT(TSPIx, TSPI_INT_TXEND, ENABLE);

	TSPI_SetBaudRate(TSPIx, clk, divider);

	init.DataDirection = TSPI_MSB_FIRST;
	init.FrameLength = 24U;
	init.CycleBetweenFrames = 0U;
	init.ClockPolarity = TSPI_CLKPOL_HIGH;
	init.ClockPhase = TSPI_CLOCKPHASE_SECONDEDGE;
	init.ShortestIdleCycle = 1U;
	init.CS0Polarity = TSPI_CS_POLARITY_NEGATIVE;
	init.CS1Polarity = TSPI_CS_POLARITY_NEGATIVE;
	init.DelayCycleNum_CS_SCLK = 1U;
	init.DelayCycleNum_Negate_CS = 1U;
	init.ParityCheck = DISABLE;
	init.Parity = TSPI_PARITY_ODD;
	init.LastBit_Hold_Time = 0U;

	TSPI_Init(TSPIx, &init);

	if (TSPIx == TSB_TSPI3)
	{
		NVIC_ClearPendingIRQ(INTSC3RX_IRQn);
		NVIC_ClearPendingIRQ(INTSC3TX_IRQn);
		NVIC_ClearPendingIRQ(INTSC3ERR_IRQn);

		/* TSPI3TXD:PC0, TSPI3CLK:PC2 */
		CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTC, ENABLE);
		TSB_PC->FR3 |= ((1 << 2u) | (1 << 0u));
		TSB_PC->CR |= ((1 << 2u) | (1 << 0u));
	}
	else if (TSPIx == TSB_TSPI2)
	{
		NVIC_ClearPendingIRQ(INTSC2RX_IRQn);
		NVIC_ClearPendingIRQ(INTSC2TX_IRQn);
		NVIC_ClearPendingIRQ(INTSC2ERR_IRQn);

		/* TSPI2TXD:PG0, TSPI2CLK:PG2 */
		CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTG, ENABLE);
		TSB_PG->FR2 |= ((1 << 2u) | (1 << 0u));
		TSB_PG->CR |= ((1 << 2u) | (1 << 0u));
	}
	else if (TSPIx == TSB_TSPI1)
	{
		NVIC_ClearPendingIRQ(INTSC1RX_IRQn);
		NVIC_ClearPendingIRQ(INTSC1TX_IRQn);
		NVIC_ClearPendingIRQ(INTSC1ERR_IRQn);

		/* TSPI1TXD:PA0, TSPI1CLK:PA2 */
		CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTA, ENABLE);
		TSB_PA->FR3 |= ((1 << 2u) | (1 << 0u));
		TSB_PA->CR |= ((1 << 2u) | (1 << 0u));
	}
	else if (TSPIx == TSB_TSPI0)
	{
		NVIC_ClearPendingIRQ(INTSC0RX_IRQn);
		NVIC_ClearPendingIRQ(INTSC0TX_IRQn);
		NVIC_ClearPendingIRQ(INTSC0ERR_IRQn);

		/* TSPI0TXD:PK3, TSPI0CLK:PK4 */
		CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTK, ENABLE);
		TSB_PK->FR3 |= ((1 << 4u) | (1 << 3u));
		TSB_PK->CR |= ((1 << 4u) | (1 << 3u));
	}
	else
	{
		/* Do nothing */
	}

	/* Setting ~SYNC port */
	PORT_DAC_SYNC_DATA_INI;
	PORT_DAC_SYNC_CR_INI;

	DacLocalWriteInit(TSPIx);

	NVIC_SetPriority(DAC_IRQ_INTTX, DAC_IRQ_LEVEL); 		/* Interruption level set */
	NVIC_ClearPendingIRQ(DAC_IRQ_INTTX);
	NVIC_EnableIRQ(DAC_IRQ_INTTX);							/* INTTX enable */

}


/**
 *********************************************************************************************
 * @brief		D/A initial output.
 *
 * @param		TSB_TSPI_TypeDef * const	TSPIx		Select the tspi channel.
 *
 * @return		none
 *********************************************************************************************
 */
static void DacLocalWriteInit(TSB_TSPI_TypeDef* const TSPIx)
{

	PORT_DAC_SYNC = 0;										/* begin tx */

	NVIC_ClearPendingIRQ(DAC_IRQ_INTTX);
	TSPIx->DR = cDAC_POWERCON;
	while (!NVIC_GetPendingIRQ(DAC_IRQ_INTTX)){ /* do nothing */ };

	PORT_DAC_SYNC = 1;										/* end tx */
}


/**
 *********************************************************************************************
 * @brief		D/A data output start.
 *
 * @param		TSB_TSPI_TypeDef * const	TSPIx		Select the tspi channel.
 *
 * @return		none
 *********************************************************************************************
 */
void DacWriteStart(TSB_TSPI_TypeDef* const TSPIx)
{
	if (DacNo >= DAC_MAX_CH)
	{
		DacNo = 0;
		DacWriteContinue(TSPIx);
	}
}


/**
 *********************************************************************************************
 * @brief		D/A data output continue.
 *
 * @param		TSB_TSPI_TypeDef * const	TSPIx		Select the tspi channel.
 *
 * @return		none
 *********************************************************************************************
 */
void DacWriteContinue(TSB_TSPI_TypeDef* const TSPIx)
{
	PORT_DAC_SYNC = 1;									/* end tx */

	if (DacNo < DAC_MAX_CH)
	{
		PORT_DAC_SYNC = 0;								/* begin tx */
		TSPIx->DR = ((DacNo << 16U) | DacData[DacNo]);
		DacNo++;
	}
}

/*********************************** END OF FILE ******************************/
