/**
 ****************************************************************************
 * @file	initial.c
 * @brief	Initialize Functions for TOSHIBA 'TMPM4K' Device Series
 * @version	V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */
#include "ipdefine.h"
#include "mcuip_drv.h"
#include "D_Para.h"

#include "ipdrv_siwdt.h"
#include "ipdrv_t32a.h"
#include "ipdrv_adc.h"
#include "ipdrv_enc.h"
#include "ipdrv_cg.h"

#define DEFINE_APP
#include "initial.h"
#undef  DEFINE_APP


/*===================================================================*
	  Constant Data Definition
 *===================================================================*/
/*  */
#define	INT4KH_TMR_CH			TSB_T32A0
#define	INT4KH_IRQ_NUMBER		INTT32A0AC_IRQn
#define INT4KH_IRQ_LEVEL		INT4KH_LEVEL
#define	TMR_DIV8_PRD(x)			(uint16_t)( (uint32_t)(x) * cIMCLK_FRQ / 8)
#define	TMR_PRD_4KH				TMR_DIV8_PRD(250)

/**
 *********************************************************************************************
 * @brief		Initialize 4kH interval Timer.
 *
 * @return		none
 *********************************************************************************************
 */
void init_Timer_interval4kH(void)
{
	T32A_InitTypeDef t32a_00_a = {0};

	 /* enable clock supply to T32A00 */
	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_T32A0, ENABLE);

	t32a_00_a.CounterMode = T32A_CNT_MOD_UP;
	t32a_00_a.ClkSrc = T32A_CLK_SRC_PRESCALER;
	t32a_00_a.ClkDiv = T32A_CLK_DIV_8;
	t32a_00_a.Compare1Value = TMR_PRD_4KH;		/* periodic time is 4kHz set to RGA1 */
	t32a_00_a.ReloadValue = 0x0000U;			/* reload register is 0x0000 */

	/* Set 16 bit mode for timer A */
	T32A_SetMode(INT4KH_TMR_CH, MODE_16);
	T32A_Enable(INT4KH_TMR_CH, TIMER_A);
	/* initialize timer A channel 0 */
	T32A_Init(INT4KH_TMR_CH, TIMER_A, &t32a_00_a);

	/* Set reload condition when counter match with RGA1 */
	T32A_SetReloadCond(INT4KH_TMR_CH, TIMER_A, T32A_COND_MATCH_COMPARE1);
	/* compare0 timing is not used in T32A_INTERVAL_TIMER mode */
	T32A_SetINTMask(INT4KH_TMR_CH, TIMER_A, T32A_INTMASK_MATCH_COMPARE0);
	/* Enable interrupt T32A Timer A channel 0 */
	NVIC_SetPriority(INT4KH_IRQ_NUMBER, INT4KH_IRQ_LEVEL);
	NVIC_ClearPendingIRQ(INT4KH_IRQ_NUMBER);
	NVIC_EnableIRQ(INT4KH_IRQ_NUMBER);

	/* counter start */
	T32A_SetRunStateBySW(INT4KH_TMR_CH, TIMER_A, T32A_START_BY_SW);
}


/**
 *********************************************************************************************
 * @brief		Initialize encoder
 *
 * @return		none
 *********************************************************************************************
 */
void init_ENCen(void)
{
/******** Setting for CH0 *********************************************/
#if defined(__USE_ENCODER_CH0)

	ENC_InitTypeDef m_enc;

	CG_SetFsysSupplyPeriphB(CG_FC_PERIPH_A_ENC, ENABLE);

	m_enc.ModeType = ENC_ENCODER_MODE;
	m_enc.PhaseType = ENC_TWO_PHASE;
	m_enc.EdgeType = ENC_RISING_EDGE;
	m_enc.ZphaseStatus = ENC_ZPHASE_ENABLE;
	m_enc.SetClearCount = ENC_CLEAR_INT;
	m_enc.UpDownType = ENC_COUNT_UP;
	m_enc.CountOperation = ENC_COUNT_CONTINUES;
	m_enc.CompareMode = ENC_COMPARE_MATCH;
	m_enc.DetectionDirection = ENC_CW_CCW_EDGE;
	m_enc.SkipDetection = ENC_DETECTION_DISABLE;
	m_enc.SelectCapture = ENC_CAPTURE_CLEAR;
	m_enc.PulseDivFactor = ENC_PULSE_DIV1;

	ENC_Init(TSB_EN0, &m_enc);
	ENC_SetCounterReload(TSB_EN0, cENC_PULSE_NUM_CH0 * cENCMULTIPLE - 1);
	ENC_SetSampleClk(TSB_EN0, ENC_SAMPLE_CLK_FSYS_8);
	ENC_SetNoiseCancelTime(TSB_EN0, (uint32_t)(cENC_NOISE_TIME_CH0 * (cIMCLK_FRQ >> ENC_SAMPLE_CLK_FSYS_8)));

	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTG, ENABLE);
	TSB_PG->FR5 |= BIT8(0, 0, 0, 0, 0, 1, 1, 1);				/* PG0,PG1,PG2 as ENC0A,ENC0B,ENC0Z */
	TSB_PG->IE  |= BIT8(0, 0, 0, 0, 0, 1, 1, 1);				/* PG0,PG1,PG2 as input,input,input */

	/* Enable ENC */
	ENC_Enable(TSB_EN0);

#endif /* __USE_ENCODER_CH0 */

}

/**
 *********************************************************************************************
 * @brief		Initialize ADC
 *
 * @return		none
 *********************************************************************************************
 */
void init_ADCen(void)
{
	AD_InitTypeDef adini;

/******** Setting for CH1 *********************************************/
#if defined(__CONTROL_MOTOR_CH1)
	adini.shunt = cSHUNT_TYPE_CH1;
	adini.iuch = cADCH_CURRENT_U_CH1;
	adini.ivch = cADCH_CURRENT_V_CH1;
	adini.iwch = cADCH_CURRENT_W_CH1;
	adini.idcch = cADCH_CURRENT_IDC_CH1;
	adini.vdcch = cADCH_VDC_CH1;
	adini.pmd_ch = cPMD1;
	adini.pints = cPINTS_B;
	adini.p_adreg0 = &Motor_ch1.drv.ADxREG0;
	adini.p_adreg1 = &Motor_ch1.drv.ADxREG1;
	adini.p_adreg2 = &Motor_ch1.drv.ADxREG2;
	adini.p_adreg3 = &Motor_ch1.drv.ADxREG3;
	IP_ADC_init(TSB_ADA, &adini);

#endif /* __CONTROL_MOTOR_CH1 */


/******** Setting for CH0 *********************************************/
#if defined(__CONTROL_MOTOR_CH0)
	adini.shunt = cSHUNT_TYPE_CH0;
	adini.iuch = cADCH_CURRENT_U_CH0;
	adini.ivch = cADCH_CURRENT_V_CH0;
	adini.iwch = cADCH_CURRENT_W_CH0;
	adini.idcch = cADCH_CURRENT_IDC_CH0;
	adini.vdcch = cADCH_VDC_CH0;
	adini.pmd_ch = cPMD0;
	adini.pints = cPINTS_A;
	adini.p_adreg0 = &Motor_ch0.drv.ADxREG0;
	adini.p_adreg1 = &Motor_ch0.drv.ADxREG1;
	adini.p_adreg2 = &Motor_ch0.drv.ADxREG2;
	adini.p_adreg3 = &Motor_ch0.drv.ADxREG3;
	IP_ADC_init(TSB_ADA, &adini);

#endif /* __CONTROL_MOTOR_CH0 */


/******** Setting for ADC Unit *********************************************/
#if defined(__ADCZ)

	/* Set PLL for ADCLK */
 /*	IP_PLL_ADCLK_init(); */ /* Set by startup. So, this code is commented out here. */

	/* Set ADC Unit A Clock */
	ADC_SetClk(TSB_ADA, ADC_SAMPLING_PERIOD_1, ADC_SCLK_1);
	/* Enable ADC Unit A */
	ADC_Enable(TSB_ADA, ADC_CLKMOD_0_5, ADC_REFSEL_VREFH, ADC_RCUT_NORMAL);
#endif /* __ADCZ */


/******** Setting for AMP Unit *********************************************/
#if defined(__USE_INAMP_CH1) || defined(__USE_INAMP_CH0)
	IP_AMP_init();
#endif /* __USE_INAMP */

}

/**
 *********************************************************************************************
 * @brief		Initialize PMD
 *
 * @return		none
 *********************************************************************************************
 */
void init_PMDen(void)
{
	PMD_InitTypeDef	pmdini;

/******** Setting for CH1 *********************************************/
#if defined(__CONTROL_MOTOR_CH1)
	pmdini.shunt = cSHUNT_TYPE_CH1;
	pmdini.poll = cPOLL_CH1;
	pmdini.polh = cPOLH_CH1;
	pmdini.pwmrate = PWMRate(cPWMPRD_CH1);
	pmdini.deadtime = DeadTime(cDEADTIME_CH1);
	pmdini.busmode = WBUF_VE;

	IP_PMD_init(TSB_PMD1, &pmdini);
#endif /* __CONTROL_MOTOR_CH1 */


/******** Setting for CH0 *********************************************/
#if defined(__CONTROL_MOTOR_CH0)
	pmdini.shunt = cSHUNT_TYPE_CH0;
	pmdini.poll = cPOLL_CH0;
	pmdini.polh = cPOLH_CH0;
	pmdini.pwmrate = PWMRate(cPWMPRD_CH0);
	pmdini.deadtime = DeadTime(cDEADTIME_CH0);
	pmdini.busmode = WBUF_VE;

	IP_PMD_init(TSB_PMD0, &pmdini);
#endif /* __CONTROL_MOTOR_CH0 */

}


/**
 *********************************************************************************************
 * @brief		Initialize VE.
 *
 * @return		none
 *********************************************************************************************
 */
void init_VEen(void)
{

/******** Setting for CH0 *********************************************/
#if defined(__CONTROL_MOTOR_CH0)

	VE_InitTypeDef	veini;

	veini.shunt = cSHUNT_TYPE_CH0;
	veini.reptime = cREPTIME_CH0;
	veini.trgmode = cTRGMODE0_INI;
	veini.tpwm = (uint16_t)(cPWMPRD_CH0 * 1e-6 * cHZ_MAX_CH0 * (1 << 16U));

	veini.idkp = (q15_t)(FIX_15 * cID_KP_CH0 * cA_MAX_CH0 / cV_MAX_CH0);
	veini.idki = (q15_t)(FIX_15 * cID_KI_CH0 * cCTRL_PRD_CH0 * cA_MAX_CH0 / cV_MAX_CH0);
	veini.idkg = cCIDKG;
	veini.iqkp = (q15_t)(FIX_15 * cIQ_KP_CH0 * cA_MAX_CH0 / cV_MAX_CH0);
	veini.iqki = (q15_t)(FIX_15 * cIQ_KI_CH0 * cCTRL_PRD_CH0 * cA_MAX_CH0 / cV_MAX_CH0);
	veini.iqkg = cCIQKG;

	veini.zerooffset = ADC_V2HEX(cSHUNT_ZERO_OFFSET_CH0);

	NVIC_SetPriority(INTVCN0_IRQn, INT_VC_LEVEL);		/* Interruption level set */
	NVIC_ClearPendingIRQ(INTVCN0_IRQn);
	NVIC_EnableIRQ(INTVCN0_IRQn);						/* INTVCNx enable */

	IP_VE_init(TSB_VE0, &veini);

#endif /* defined(__CONTROL_MOTOR_CH0) */

}

/**
 *********************************************************************************************
 * @brief		Initialize WDT
 *
 * @return		none
 *********************************************************************************************
 */
void init_WDT(void)
{
	CG_SetFsysSupplyPeriphB(CG_FC_PERIPH_SIWDT, ENABLE);
	SIWDT_SetClockSource(SIWDT_CLOCK_SOURCE_FSYS_DIVIDE_FOUR);
	// SIWDT_SetDetectTime(SIWDT_DETECT_TIME_EXP_29);	/* 25sec */
	// SIWDT_SetDetectTime(SIWDT_DETECT_TIME_EXP_27);	/* 7sec */
	SIWDT_SetDetectTime(SIWDT_DETECT_TIME_EXP_25);	/* 2sec */
	SIWDT_SetClearWindow(SIWDT_CLEAR_WINDOW_NOT_SPECIFIED);
	/* SIWDT_SetClearWindow(SIWDT_CLEAR_WINDOW_LATTER_A_HALF); */
	SIWDT_SetOperationAfterDetectMalFunction(SIWDT_OPERATION_INTERRUPT);
	SIWDT_Enable();
	SIWDT_SetProtectMode(SIWDT_NO_PROTECTION);
}

/*********************************** END OF FILE ******************************/
