/**
 ****************************************************************************
 * @file	interrupt.c
 * @brief	Interrupt Functions for All Soft
 * @version	V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */
#include <stdlib.h>

#include "ipdefine.h"
#include "mcuip_drv.h"

#include "D_Driver.h"
#include "usercon.h"

#define DEFINE_APP
#include "interrupt.h"
#undef	DEFINE_APP

//#define MOTOR_OUTPUT	0
#define MOTOR_OUTPUT	1

/* for debug */
#if	defined(__MOTOR_DBGOUT_IDET)

	#define	DEBUG_IDET_ERROR_VE(x)		debug_idet_error_ve(x)

	void	debug_idet_error_ve(TSB_VE_TypeDef* const VEx)
	{
		if ((VEx->MCTLF & 0x0010) != 0)
		{
			TSB_PA_DATA_PA0 = 1;
		}
		else
		{
			TSB_PA_DATA_PA0 = 0;
		}
	}

#else

	#define	DEBUG_IDET_ERROR_VE(x)

#endif

/**
 *********************************************************************************************
 * @brief		INT_interval_4kHz
 *
 * @return		none
 *********************************************************************************************
 */
void INT_interval_4kHz(void)
{
	M_Main_Counter++;									/* Count for main routine Period */

}

/**
 ********************************************************************************************
 * @brief		Vector Control Interruption by VE for ch0.
 *
 * @param[in]	vector_t* const			_motor	.
 * @param[in]	TSB_VE_TypeDef* const	VEx		.
 *
 * @return		none
 ********************************************************************************************
 */
#if defined(__CONTROL_MOTOR_CH0)
void INT_VectorControlByVE_CH0(vector_t* const _motor , TSB_VE_TypeDef* const VEx)
{

	/* LED5 ON */
	TSB_PG_DATA_PG4 = 1;

	DEBUG_IDET_ERROR_VE(VEx);

#if (__FIXED_VDC_CH0 == 1)								/* 1: Vdc from constant data */
	/***** Set Vdc data to VE *****/
	VE_SetVDCreg(VEx, (q15_t)(FIX_15 * cVDC_CH0 / cV_MAX_CH0));
#endif

	/***** Get data from VE register *****/
	VE_GetdataFromVEreg(VEx, _motor);

	/***** Check over current *****/
	VE_GetPhaseCurrent(VEx, &_motor->drv.Ia, &_motor->drv.Ib, &_motor->drv.Ic);
	if (D_Check_OverCurrent(_motor->para.err_ovc, _motor->drv.Ia, _motor->drv.Ib, _motor->drv.Ic))
	{
		_motor->drv.state.flg.emg_S = SET;
		_motor->stage.itr = ciEmergency;
	}

	/***** Check error voltage *****/
	if ((_motor->drv.Vdc < (q15_t)(FIX_15 * cVDC_MINLIM_CH0 / cV_MAX_CH0)) || ((q15_t)(FIX_15 * cVDC_MAXLIM_CH0 / cV_MAX_CH0) < _motor->drv.Vdc))
	{
		_motor->drv.state.flg.emg_DC = SET;
		_motor->stage.itr = ciEmergency;
	}

	if (_motor->drv.state.all == 0)						/* No error? */
	{
		/***** Detect Rotor Position *****/
		D_Detect_Rotor_Position(_motor);

#if defined(__USE_ENCODER_CH0)
		/***** Encoder *****/
		H_Encoder(_motor, TSB_EN0);
#endif /* __USE_ENCODER_CH0 */

		/***** Control Speed *****/
		_motor->drv.spdprd++;
		if (_motor->drv.spdprd >= cSPD_PI_PRD_CH0)
		{
			_motor->drv.spdprd = 0;
			D_Control_Speed(_motor);
		}
	}

	/**** Set data to VE register *****/
	/* Set Shift pwm change speed. for 1shunt */
	VE_SetSpwmMode(VEx, _motor->drv.command.spwm);

	/* Set modulation type */
	VE_SetModulType(VEx, _motor->drv.command.modul);

	switch (_motor->stage.itr)
	{
			/*--- Stop Stage -----------------------------*/
		case ciStop:
			VE_SetdataToVEreg_Stop(VEx, _motor);
			break;

			/*--- Bootstrap Stage ------------------------*/
		case ciBootstrap:
			VE_SetdataToVEreg_Bootstrap(VEx, _motor);
			break;

			/*--- Position Stage (Current type) ----------*/
		case ciInitposition_i:
			VE_SetdataToVEreg_Initposition_i(VEx, _motor);
			break;

			/*--- Position Stage (Voltage type) ----------*/
		case ciInitposition_v:
			VE_SetdataToVEreg_Initposition_v(VEx, _motor);
			break;

			/*--- Force Stage (Current type) -------------*/
		case ciForce_i:
			VE_SetdataToVEreg_Force_i(VEx, _motor);
			break;

			/*--- Force Stage (Voltage type) -------------*/
		case ciForce_v:
			VE_SetdataToVEreg_Force_v(VEx, _motor);
			break;

			/*--- ChangeUp Stage -------------------------*/
		case ciChange_up:
			VE_SetdataToVEreg_Change_up(VEx, _motor);
			break;

			/*--- Steady_A Stage ---------------------------*/
		case ciSteady_A:
			VE_SetdataToVEreg_Steady_A(VEx, _motor);
			break;

			/*--- Emergency Stage ------------------------*/
		case ciEmergency:
			VE_SetdataToVEreg_Emergency(VEx, _motor);
			break;
			/*--- Fail-safe ------------------------*/
		default:
			VE_SetdataToVEreg_Stop(VEx, _motor);
			break;
	}

#if defined(__USE_DAC)
	UiOutDataStartByVE(0);							/* Dac output for debug */
#endif /* __USE_DAC */

	VE_Start(VEx);

	/* LED5 OFF */
	TSB_PG_DATA_PG4 = 0;

}
#endif /* defined(__CONTROL_MOTOR_CH0) */

/**
 ********************************************************************************************
 * @brief		Vector Control Interruption by VE for ch1.
 *
 * @param[in]	vector_t* const			_motor	.
 * @param[in]	TSB_VE_TypeDef* const	VEx		.
 *
 * @return		none
 ********************************************************************************************
 */
#if defined(__CONTROL_MOTOR_CH1)
void INT_VectorControlByVE_CH1(vector_t* const _motor , TSB_VE_TypeDef* const VEx)
{
	/* LED5 ON */
	TSB_PG_DATA_PG4 = 1;

	DEBUG_IDET_ERROR_VE(VEx);

#if (__FIXED_VDC_CH1 == 1)								/* 1: Vdc from constant data */
	/***** Set Vdc data to VE *****/
	VE_SetVDCreg(VEx, (q15_t)(FIX_15 * cVDC_CH1 / cV_MAX_CH1));
#endif

	/***** Get data from VE register *****/
	VE_GetdataFromVEreg(VEx, _motor);

	/***** Check over current *****/
	VE_GetPhaseCurrent(VEx, &_motor->drv.Ia, &_motor->drv.Ib, &_motor->drv.Ic);
	if (D_Check_OverCurrent(_motor->para.err_ovc, _motor->drv.Ia, _motor->drv.Ib, _motor->drv.Ic))
	{
		_motor->drv.state.flg.emg_S = SET;
		_motor->stage.itr = ciEmergency;
	}

	/***** Check error voltage *****/
	if ((_motor->drv.Vdc < (q15_t)(FIX_15 * cVDC_MINLIM_CH1 / cV_MAX_CH1)) || ((q15_t)(FIX_15 * cVDC_MAXLIM_CH1 / cV_MAX_CH1) < _motor->drv.Vdc))
	{
		_motor->drv.state.flg.emg_DC = SET;
		_motor->stage.itr = ciEmergency;
	}

	if (_motor->drv.state.all == 0)						/* No error? */
	{
		/***** Detect Rotor Position *****/
		D_Detect_Rotor_Position(_motor);

#if defined(__USE_ENCODER_CH1)
		/***** Encoder *****/
		H_Encoder(_motor, TSB_EN1);
#endif /* __USE_ENCODER_CH1 */

		/***** Control Speed *****/
		_motor->drv.spdprd++;
		if (_motor->drv.spdprd >= cSPD_PI_PRD_CH1)
		{
			_motor->drv.spdprd = 0;
			D_Control_Speed(_motor);
		}
	}

	/**** Set data to VE register *****/
	/* Set Shift pwm change speed. for 1shunt */
	VE_SetSpwmMode(VEx, _motor->drv.command.spwm);

	/* Set modulation type */
	VE_SetModulType(VEx, _motor->drv.command.modul);

	switch (_motor->stage.itr)
	{
			/*--- Stop Stage -----------------------------*/
		case ciStop:
			VE_SetdataToVEreg_Stop(VEx, _motor);
			break;

			/*--- Bootstrap Stage ------------------------*/
		case ciBootstrap:
			VE_SetdataToVEreg_Bootstrap(VEx, _motor);
			break;

			/*--- Position Stage (Current type) ----------*/
		case ciInitposition_i:
			VE_SetdataToVEreg_Initposition_i(VEx, _motor);
			break;

			/*--- Position Stage (Voltage type) ----------*/
		case ciInitposition_v:
			VE_SetdataToVEreg_Initposition_v(VEx, _motor);
			break;

			/*--- Force Stage (Current type) -------------*/
		case ciForce_i:
			VE_SetdataToVEreg_Force_i(VEx, _motor);
			break;

			/*--- Force Stage (Voltage type) -------------*/
		case ciForce_v:
			VE_SetdataToVEreg_Force_v(VEx, _motor);
			break;

			/*--- ChangeUp Stage -------------------------*/
		case ciChange_up:
			VE_SetdataToVEreg_Change_up(VEx, _motor);
			break;

			/*--- Steady_A Stage ---------------------------*/
		case ciSteady_A:
			VE_SetdataToVEreg_Steady_A(VEx, _motor);
			break;

			/*--- Emergency Stage ------------------------*/
		case ciEmergency:
			VE_SetdataToVEreg_Emergency(VEx, _motor);
			break;
			/*--- Fail-safe ------------------------*/
		default:
			VE_SetdataToVEreg_Stop(VEx, _motor);
			break;
	}

#if defined(__USE_DAC)
	UiOutDataStartByVE(1);									/* Dac output for debug */
#endif /* __USE_DAC */

	VE_Start(VEx);

	/* LED5 OFF */
	TSB_PG_DATA_PG4 = 0;

}
#endif /* defined(__CONTROL_MOTOR_CH1) */


/*********************************** END OF FILE ******************************/
