/**
 ****************************************************************************
 * @file	mcuip_drv.c
 * @brief	MCU IP driver for A-VE Plus
 * @version	V1.0.0
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */
#include "ipdefine.h"

#include "ipdrv_adc.h"
#include "ipdrv_cg.h"

#define DEFINE_APP
#include "mcuip_drv.h"
#undef DEFINE_APP




/**
 *********************************************************************************************
 * @brief		Get EMGstatus
 *
 * @param		TSB_PMD_TypeDef *	PMDx	Select the PMD channel.
 *
 * @return		emg_status_e				EMG status
 *********************************************************************************************
 */
emg_status_e	PMD_GetEMG_Status(TSB_PMD_TypeDef* const PMDx)
{

	return ((emg_status_e)(*((__I uint32_t*)BITBAND_PERI(&PMDx->EMGSTA, B_EMGST))));

}

/**
 *********************************************************************************************
 * @brief		Release EMG protection.
 *
 * @param		TSB_PMD_TypeDef *	PMDx	Select the PMD channel.
 *
 * @return		none
 *********************************************************************************************
 */
void	PMD_ReleaseEMG_Protection(TSB_PMD_TypeDef* const PMDx)
{
	emg_status_e	emgst;

	emgst = PMD_GetEMG_Status(PMDx);

	if (emgst == cEMGProtected)
	{
		if ((*((__I uint32_t*)BITBAND_PERI(&PMDx->EMGSTA, B_EMGI))) == 1)
		{
			if (PMDx == TSB_PMD1)
			{
			}
			else
			{
				TSB_VE0->EMGRS = 1;								/*Release EMG protection */
			}
		}
	}

}



/**
 ********************************************************************************************
 * @brief		Set OutputMode Register.
 *
 * @param[in]	TSB_PMD_TypeDef* const	PMDx	.
 * @param[in]	uint32_t				_mdout	.
 *
 * @return		none
 ********************************************************************************************
 */
void PMD_SetOutputMode(TSB_PMD_TypeDef* const PMDx, uint32_t _mdout)
{
	PMDx->MDOUT = _mdout;
	
}


/**
 *********************************************************************************************
 * @brief		VE start
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_Start(TSB_VE_TypeDef* const VEx)
{
	VEx->CPURUNTRG = VE_START;

}

/**
 *********************************************************************************************
 * @brief		VE_GetTask
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 *
 * @return		uint32_t						Task No.
 *********************************************************************************************
 */
uint32_t	VE_GetTask(const TSB_VE_TypeDef* const VEx)
{
	return((VEx->SCHTASKRUN >> 1U) & 0x0000000f);
}

/**
 *********************************************************************************************
 * @brief		Get Phase Current from VE register.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		q15_t*					_ia		U-phase current value store address.
 * @param		q15_t*					_ib		V-phase current value store address.
 * @param		q15_t*					_ic		W-phase current value store address.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_GetPhaseCurrent(TSB_VE_TypeDef* const VEx, q15_t* _ia, q15_t* _ib, q15_t* _ic)
{
	*_ia = VEx->IA >> 16;
	*_ib = VEx->IB >> 16;
	*_ic = VEx->IC >> 16;
}

/**
 *********************************************************************************************
 * @brief		Get current adc data from VE register.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		uint32_t*				_adc_ia	U-phase current ad value store address.
 * @param		uint32_t*				_adc_ib	V-phase current ad value store address.
 * @param		uint32_t*				_adc_ic	W-phase current ad value store address.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_GetCurrentAdcData(TSB_VE_TypeDef* const VEx, uint32_t* _adc_ia, uint32_t* _adc_ib, uint32_t* _adc_ic)
{

	*_adc_ia = VEx->IAADC;
	*_adc_ib = VEx->IBADC;
	*_adc_ic = VEx->ICADC;

}

/**
 *********************************************************************************************
 * @brief		Get data from VE registers.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_GetdataFromVEreg(TSB_VE_TypeDef* const VEx, vector_t* const _motor)
{
	_motor->drv.Vdc = VEx->VDC;									/* Read Vdc */
	_motor->drv.Vd.word = VEx->VD;								/* Read Vd */
	_motor->drv.Vq.word = VEx->VQ;								/* Read Vd */

	if (_motor->drv.idetect_error == 0)
	{
		_motor->drv.Id = (q15_t)(VEx->ID >> 16);				/* Read Id */
		_motor->drv.Iq = (q15_t)(VEx->IQ >> 16);				/* Read Iq */
	}
	else
	{
		VEx->ID = (q31_t)_motor->drv.Id << 16;					/* Set Id */
		VEx->IQ = (q31_t)_motor->drv.Iq << 16;					/* Set Iq */
	}
}

/**
 *********************************************************************************************
 * @brief		VE_GetPWM_DutyU
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 *
 * @return		uint32_t						U Phase duty.
 *********************************************************************************************
 */
uint32_t	VE_GetPWM_DutyU(TSB_VE_TypeDef* const VEx)
{
	return(VEx->CMPU);
}

/**
 *********************************************************************************************
 * @brief		VE_GetPWM_DutyV
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 *
 * @return		uint32_t						V Phase duty.
 *********************************************************************************************
 */
uint32_t	VE_GetPWM_DutyV(TSB_VE_TypeDef* const VEx)
{
	return(VEx->CMPV);
}

/**
 *********************************************************************************************
 * @brief		VE_GetPWM_DutyW
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 *
 * @return		uint32_t						W Phase duty.
 *********************************************************************************************
 */
uint32_t	VE_GetPWM_DutyW(TSB_VE_TypeDef* const VEx)
{
	return(VEx->CMPW);
}

/**
 *********************************************************************************************
 * @brief		Get medium duty from VE registers.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Select the VE channel.
 *
 * @return		uint32_t						Medium duty value.
 *********************************************************************************************
 */
uint32_t	VE_GetPWM_DutyMed(TSB_VE_TypeDef* const VEx)
{
	uint32_t	a,b,c,ret;

	a = VEx->CMPU;
	b = VEx->CMPV;
	c = VEx->CMPW;

	if (a >= b)
	{
		ret = (c < b) ? b : (c > a) ? a : c;
	}
	else
	{
		ret = (c < a) ? a : (c > b) ? b : c;
	}

	return ret;

}

/**
 *********************************************************************************************
 * @brief		Get medium duty from VE registers.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Select the VE channel.
 *
 * @return		uint32_t						Medium duty value.
 *********************************************************************************************
 */
uint32_t	VE_GetPWM_DutyMin(TSB_VE_TypeDef* const VEx)
{
	uint32_t	a,b,c,ret;

	a = VEx->CMPU;
	b = VEx->CMPV;
	c = VEx->CMPW;

	if (a >= b)
	{
		ret = (c < b) ? c : b;
	}
	else
	{
		ret = (c < a) ? c : a;
	}

	return ret;

}

/**
 *********************************************************************************************
 * @brief		VE_GetPWM_Modulation
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 *
 * @return		int								PWM modulation 0:3Phase, 1:2Phase
 *********************************************************************************************
 */
int	VE_GetPWM_Modulation(TSB_VE_TypeDef* const VEx)
{
	int	ret = 0;

	if ((VEx->FMODE & (1 << B_C2PEN)) != 0)
	{
		ret = 1;
	}

	return(ret);

}

/**
 *********************************************************************************************
 * @brief		VE_GetSector
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 *
 * @return		uint32_t						Sector
 *********************************************************************************************
 */
uint32_t	VE_GetSector(TSB_VE_TypeDef* const VEx)
{
	return(VEx->SECTOR);
}

/**
 *********************************************************************************************
 * @brief		VE_GetShiftPWMState
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 *
 * @return		int								PWM Shift state 0:Normal PWM, 1;Shift PWM
 *********************************************************************************************
 */
int	VE_GetShiftPWMState(TSB_VE_TypeDef* const VEx)
{
	int	ret = 0;

	if ((VEx->MCTLF & (1 << B_LAVF)) != 0)
	{
		ret = 1;
	}

	return(ret);
}

/**
 *********************************************************************************************
 * @brief		VE_GetOutputMode
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 *
 * @return		int	
 *********************************************************************************************
 */
int	VE_GetOutputMode(TSB_VE_TypeDef* const VEx)
{
	return(((VEx->MODE >> 2U) & 0x00000003));
}



/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Stop stage.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Stop(TSB_VE_TypeDef* const VEx, const vector_t* const _motor)
{

	VEx->TASKAPP = SET_TAPP(TAPP_OUT_CON);						/* VE start from output control */
	VEx->ACTSCH = SET_ACTSCH(SCH_9);							/* Schedule 9 */

	VEx->MODE = VEMODE_STOP;									/* Output OFF */

	VEx->ID = 0;
	VEx->IQ = 0;
	VEx->VD = 0;
	VEx->VQ = 0;
	VEx->VDIH = 0;												/* Clear Current PI Control Integral Term Registers */
	VEx->VDILH = 0;
	VEx->VQIH = 0;
	VEx->VQILH = 0;
	VEx->IDREF = 0;
	VEx->IQREF = 0;
	VEx->THETA = 0;
	VEx->OMEGA = 0;
	VEx->SECTOR = 0;
	VEx->SECTORM = 0;
	VEx->MINPLS = 0;

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Bootstrap stage.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Bootstrap(TSB_VE_TypeDef* const VEx, const vector_t* const _motor)
{

	VEx->TASKAPP = SET_TAPP(TAPP_SINCOS);						/* VE start from SIN/COS */
	VEx->ACTSCH = SET_ACTSCH(SCH_4);							/* Schedule 4 */

	VEx->MODE = VEMODE_BOOT;									/* Output ON */

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Initposition stage for current control.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Initposition_i(TSB_VE_TypeDef* const VEx, const vector_t* const _motor)
{

	VEx->TASKAPP = SET_TAPP(TAPP_CRT_CON);						/* VE start from current control */
	VEx->ACTSCH = SET_ACTSCH(SCH_1);							/* Schedule 1 */

	VEx->MODE = VEMODE_INITPOS;									/* Output ON */

	VEx->IDREF = _motor->drv.Id_ref;							/* Set Id */
	VEx->IQREF = _motor->drv.Iq_ref; 							/* Set Iq */
	VEx->THETA = _motor->drv.theta.half[1];
	VEx->OMEGA = _motor->drv.omega.half[1];
	if (_motor->drv.command.spwm == 0)
	{
		VEx->MINPLS = _motor->drv.chkpls;
	}
	else
	{
		VEx->MINPLS = _motor->drv.chkpls / 2;
	}
}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Initposition stage for voltage control.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Initposition_v(TSB_VE_TypeDef* const VEx, const vector_t* const _motor)
{
	VEx->TASKAPP = SET_TAPP(TAPP_SINCOS);						/* VE start from SIN/COS */
	VEx->ACTSCH = SET_ACTSCH(SCH_4);							/* Schedule 4 */

	VEx->MODE = VEMODE_INITPOS;									/* Output ON */

	VEx->VD = _motor->drv.Vd_out;								/* Vd */
	VEx->VQ = 0;
	VEx->THETA = _motor->drv.theta.half[1];
	VEx->OMEGA = _motor->drv.omega.half[1];
	if (_motor->drv.command.spwm == 0)
	{
		VEx->MINPLS = _motor->drv.chkpls;
	}
	else
	{
		VEx->MINPLS = _motor->drv.chkpls / 2;
	}

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Force stage for current control.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Force_i(TSB_VE_TypeDef* const VEx, const vector_t* const _motor)
{
	VEx->TASKAPP = SET_TAPP(TAPP_CRT_CON);						/* VE start from current control */
	VEx->ACTSCH = SET_ACTSCH(SCH_1);							/* Schedule 1 */

	VEx->MODE = VEMODE_FORCE;									/* Output ON */

	VEx->IDREF = _motor->drv.Id_ref;							/* Set Id */
	VEx->IQREF = _motor->drv.Iq_ref; 							/* Set Iq */
	VEx->THETA = _motor->drv.theta.half[1];
	VEx->OMEGA = _motor->drv.omega.half[1];
	if (_motor->drv.command.spwm == 0)
	{
		VEx->MINPLS = _motor->drv.chkpls;
	}
	else
	{
		VEx->MINPLS = _motor->drv.chkpls / 2;
	}

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Force stage for voltage control.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Force_v(TSB_VE_TypeDef* const VEx, const vector_t* const _motor)
{
	VEx->TASKAPP = SET_TAPP(TAPP_SINCOS);						/* VE start from SIN/COS */
	VEx->ACTSCH = SET_ACTSCH(SCH_4);							/* Schedule 4 */

	VEx->MODE = VEMODE_FORCE;									/* Output ON */

	VEx->VD = _motor->drv.Vd_out;								/* Vd */
	VEx->VDIH = _motor->drv.Vd_out;								/* Vd Integral value initialization */
	VEx->VQ = 0; 												/* Vq = 0 */
	VEx->THETA = _motor->drv.theta.half[1];						/* Theta set */
	VEx->OMEGA = _motor->drv.omega.half[1];						/* Omega set */
	if (_motor->drv.command.spwm == 0)
	{
		VEx->MINPLS = _motor->drv.chkpls;
	}
	else
	{
		VEx->MINPLS = _motor->drv.chkpls / 2;
	}

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Change_up stage.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Change_up(TSB_VE_TypeDef* const VEx, const vector_t* const _motor)
{
	VEx->TASKAPP = SET_TAPP(TAPP_CRT_CON);						/* VE start from current control */
	VEx->ACTSCH = SET_ACTSCH(SCH_1);							/* Schedule 1 */

	VEx->MODE = VEMODE_CHGUP;									/* Output ON */

	VEx->IDREF = _motor->drv.Id_ref;							/* Set Id */
	VEx->IQREF = _motor->drv.Iq_ref; 							/* Set Iq */
	VEx->THETA = _motor->drv.theta.half[1];
	VEx->OMEGA = _motor->drv.omega.half[1];
	if (_motor->drv.command.spwm == 0)
	{
		VEx->MINPLS = _motor->drv.chkpls;
	}
	else
	{
		VEx->MINPLS = _motor->drv.chkpls / 2;
	}

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Steady_A stage.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Steady_A(TSB_VE_TypeDef* const VEx, const vector_t* const _motor)
{
	VEx->TASKAPP = SET_TAPP(TAPP_CRT_CON);						/* VE start from current control */
	VEx->ACTSCH = SET_ACTSCH(SCH_1);							/* Schedule 1 */

	VEx->MODE = VEMODE_STEADY;									/* Output ON */

	VEx->IDREF = _motor->drv.Id_ref;							/* Id set */
	VEx->IQREF = _motor->drv.Iq_ref;							/* Iq set */
	VEx->THETA = _motor->drv.theta.half[1];						/* Theta set */
	VEx->OMEGA = _motor->drv.omega.half[1];						/* Omega set */
	if (_motor->drv.command.spwm == 0)
	{
		VEx->MINPLS = _motor->drv.chkpls;
	}
	else
	{
		VEx->MINPLS = _motor->drv.chkpls / 2;
	}

}

/**
 *********************************************************************************************
 * @brief		Set data to VE registers in Emergency stage.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		vector_t* const			_motor	Structure address of vector control variable.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetdataToVEreg_Emergency(TSB_VE_TypeDef* const VEx, const vector_t* const _motor)
{
	VEx->TASKAPP = SET_TAPP(TAPP_OUT_CON);						/* VE start from output control */
	VEx->ACTSCH = SET_ACTSCH(SCH_9);							/* Schedule 9 */

	VEx->MODE = VEMODE_EMG;										/* Output OFF */

}

/**
 *********************************************************************************************
 * @brief		Set zero current data to VE register.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		uint32_t				_z_ia	U Phase value.
 * @param		uint32_t				_z_ib	V Phase value.
 * @param		uint32_t				_z_ic	W Phase value.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetZeroCurrentData(TSB_VE_TypeDef* const VEx, uint32_t _z_ia, uint32_t _z_ib, uint32_t _z_ic)
{
	VEx->IAO = _z_ia;
	VEx->IBO = _z_ib;
	VEx->ICO = _z_ic;

}

/**
 *********************************************************************************************
 * @brief		Set VEVDC reg.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		q15_t					_dat	Power supply voltage(Q15)
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetVDCreg(TSB_VE_TypeDef* const VEx, q15_t _dat)
{
	VEx->VDC = _dat;
}

/**
 *********************************************************************************************
 * @brief		Set shift pwm_mode.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		uint8_t					_dat	Shift pwm mode.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetSpwmMode(TSB_VE_TypeDef* const VEx, uint8_t _dat)
{
	(*((__IO uint32_t*)BITBAND_PERI(&VEx->FMODE, B_SPWMEN))) = _dat;
}

/**
 *********************************************************************************************
 * @brief		Set modulation type.
 *
 * @param		TSB_VE_TypeDef* const	VEx		Set the VE address.
 * @param		uint8_t					_dat	modulation type.
 *
 * @return		none
 *********************************************************************************************
 */
void	VE_SetModulType(TSB_VE_TypeDef* const VEx, uint8_t _dat)
{
	(*((__IO uint32_t*)BITBAND_PERI(&VEx->FMODE, B_C2PEN))) = _dat;
}

/**
 *********************************************************************************************
 * @brief		PMD IP Initialize
 *
 * @param		TSB_PMD_TypeDef* const	PMDx		Select the PMD channel.
 * @param		PMD_InitTypeDef* const	_initdata	The structure containing basic PMD configuration.
 *
 * @return		none
 *********************************************************************************************
 */
void IP_PMD_init(TSB_PMD_TypeDef* const PMDx, PMD_InitTypeDef* const _initdata)
{
	if (PMDx == TSB_PMD1) {
		CG_SetFsysSupplyPeriphB(CG_FC_PERIPH_PMD1, ENABLE);
	}
	else
	{
		CG_SetFsysSupplyPeriphB(CG_FC_PERIPH_PMD0, ENABLE);
	}

	PMDx->MODESEL = WBUF_BUS;

	/***** active setting *****/
	PMDx->MDPOT = cMDPOT_INI;
	if (_initdata->poll == 1)
	{
		PMDx->MDPOT |= (1U << B_POLL);
	}
	else
	{
		PMDx->MDPOT &= ~(1U << B_POLL);
	}
	if (_initdata->polh == 1)
	{
		PMDx->MDPOT |= (1U << B_POLH);
	}
	else
	{
		PMDx->MDPOT &= ~(1U << B_POLH);
	}

	PMDx->MDOUT = cMDOUT_OFF;									/* Output off */
	PMDx->CMPU = 0;
	PMDx->CMPV = 0;
	PMDx->CMPW = 0;

	/***** EMG setting *****/
#if (1)
	PMDx->EMGCR = cEMG_ENA; 									/* EMG Enable */
	PMDx->EMGCR = cEMG_Release; 								/* EMG Release */
#else
	PMDx->EMGREL = 0x5A;										/* Disable code */
	PMDx->EMGREL = 0xA5;
	PMDx->EMGCR = cEMG_DIS; 									/* EMG Disable */
#endif

	/* OVV disable set */
	PMDx->EMGREL = 0x5A;										/* Disable code */
	PMDx->EMGREL = 0xA5;
	PMDx->OVVCR = cOVV_DIS;

	PMDx->PORTMD = cPORTMD_INI;									/* Tool break */
	PMDx->MDCR = cMDCR_INI;

	PMDx->RATE = _initdata->pwmrate;							/* PWM rate */
	PMDx->DTR = _initdata->deadtime; 							/* Dead timewr set */
	PMDx->MDEN = 0x01;											/* PMD enable (Set after set MDOUT,MDPOT) */

	/***** port setting *****/
	if (PMDx == TSB_PMD1)
	{
		CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTG, ENABLE);
        TSB_PG->FR6 |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD1 PORT output Enable */
		TSB_PG->CR  |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD1 PORT output Enable */

		if ((*((__IO uint32_t*)BITBAND_PERI(&PMDx->EMGCR, B_EMGEN))) == 1)	/* EMG enable? */
		{
            CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTF, ENABLE);
			TSB_PF->FR7 |= BIT8(0, 0, 0, 0, 0, 0, 0, 1);		/* PMD1 PORT Enable */
			TSB_PF->IE  |= BIT8(0, 0, 0, 0, 0, 0, 0, 1); 		/* PMD1 PORT Enable */
		}

	#if 0 /* M4K4A do not have OVV CH1. */
		if ((*((__IO uint32_t*)BITBAND_PERI(&PMDx->OVVCR, B_OVVEN))) == 1)	/* OVV enable? */
		{
		}
	#endif

	#if defined(__MOTOR_PMDDBG)	/* for debug */
	/* PMD DBG for CH1 */
		CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTB, ENABLE);
		TSB_PB_CR_PB1C = SET;
		TSB_PB_FR7_PB1F7 = SET;

	//	PMDx->DBGOUTCR = ((0x3FU << 16) | (1U << 1)  | (1U << 0));	/* PMD trigger timing monitor */
	//	PMDx->DBGOUTCR = ((0x03U << 16) | (1U << 1)  | (1U << 0));	/* PMD trigger 0,1 timing monitor for 1shunt */
	//	PMDx->DBGOUTCR = ((1U << 8) | (2U << 1)  | (1U << 0));	/* INTPWMx PWM Interrupt monitor */
	//	PMDx->DBGOUTCR = ((1U << 3) | (2U << 1)  | (1U << 0));	/* INTADxPA  Interrupt monitor */
	//	PMDx->DBGOUTCR = DBGOUT_ENA_VE;		/* VE Task timing monitor */
		PMDx->DBGOUTCR = DBGOUT_ENA_ADC;	//((0x03U << 16) | (1U << 0));	/* ADC convert timing monitor */
	#endif

	}
	else if (PMDx == TSB_PMD0)
	{
		CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTJ, ENABLE);
		TSB_PJ->FR5 |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD0 PORT output Enable */
		TSB_PJ->CR  |= BIT8(0, 0, 1, 1, 1, 1, 1, 1); 			/* PMD0 PORT output Enable */
		if ((*((__IO uint32_t*)BITBAND_PERI(&PMDx->EMGCR, B_EMGEN))) == 1)	/* EMG enable? */
		{
			CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTD, ENABLE);
			TSB_PD->FR5 |= BIT8(0, 1, 0, 0, 0, 0, 0, 0); 		/* PMD0 PORT Enable */
			TSB_PD->IE  |= BIT8(0, 1, 0, 0, 0, 0, 0, 0); 		/* PMD0 PORT Enable */
		}

		if ((*((__IO uint32_t*)BITBAND_PERI(&PMDx->OVVCR, B_OVVEN))) == 1)	/* OVV enable? */
		{
			TSB_PJ->FR5 |= BIT8(1, 0, 0, 0, 0, 0, 0, 0); 		/* PMD0 PORT Enable */
			TSB_PJ->IE  |= BIT8(1, 0, 0, 0, 0, 0, 0, 0); 		/* PMD0 PORT Enable */
		}

	#if defined(__MOTOR_PMDDBG)	/* for debug */
	/* PMD DBG for CH0 */
		CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTB, ENABLE);
		TSB_PB_CR_PB0C = SET;
		TSB_PB_FR7_PB0F7 = SET;

	//	PMDx->DBGOUTCR = ((0x3FU << 16) | (1U << 1)  | (1U << 0));	/* PMD trigger timing monitor */
	//	PMDx->DBGOUTCR = ((0x03U << 16) | (1U << 1)  | (1U << 0));	/* PMD trigger 0,1 timing monitor for 1shunt */
	//	PMDx->DBGOUTCR = ((1U << 8) | (2U << 1)  | (1U << 0));	/* INTPWMx PWM Interrupt monitor */
	//	PMDx->DBGOUTCR = ((1U << 3) | (2U << 1)  | (1U << 0));	/* INTADxPA  Interrupt monitor */
	//	PMDx->DBGOUTCR = DBGOUT_ENA_VE;		/* VE Task timing monitor */
		PMDx->DBGOUTCR = DBGOUT_ENA_ADC;	//((0x03U << 16) | (1U << 0));	/* ADC convert timing monitor */
	#endif

	}

	/***** Release EMG protection *****/
	if ((*((__I uint32_t*)BITBAND_PERI(&PMDx->EMGSTA, B_EMGI))) == 1)
	{
		(*((__O uint32_t*)BITBAND_PERI(&PMDx->EMGCR, B_EMGRS))) = 1;
	}


	/***** Triger setting *****/
	PMDx->TRGCMP0 = 0; 											/* TRG position clear */
	PMDx->TRGCMP1 = 0;
	PMDx->TRGCMP2 = 0;
	PMDx->TRGCMP3 = 0;

	if (_initdata->shunt == c3shunt)
	{
		PMDx->TRGCR = cTRG_3SHUNT;
		PMDx->TRGMD = cTRGMD_3SHUNT;
	}
	else if (_initdata->shunt == c1shunt)
	{
		PMDx->TRGCR = cTRG_1SHUNT;
		PMDx->TRGMD = cTRGMD_1SHUNT;
	}

	PMDx->TRGSEL = 0;											/* TRG No = Sector = 0 */

	PMDx->TRGSYNCR = cTRGSYNCR_INI;

	PMDx->MODESEL = _initdata->busmode;
	
}

/**
 *********************************************************************************************
 * @brief		IP_VE_init
 *
 * @param		TSB_VE_TypeDef* const	VEx			Select the VE channel.
 * @param		VE_InitTypeDef* const	_initdata	The structure containing basic VE configuration
 *
 * @return		none
 *********************************************************************************************
 */
void IP_VE_init(TSB_VE_TypeDef* const VEx, VE_InitTypeDef* const _initdata)
{
	CG_SetFsysSupplyPeriphB(CG_FC_PERIPH_A_VE_P, ENABLE);

	/*********************** VE Control Register ********************************/
	VEx->EN = 0x01; 											/* VE Enable */

	VEx->TASKAPP = SET_TAPP(TAPP_OUT_CON);						/* VE start from output control */
	VEx->ACTSCH = SET_ACTSCH(SCH_9); 							/* Schedule 9 */

	VEx->REPTIME = SET_REPTIME(_initdata->reptime);
	VEx->TRGMODE = SET_TRGMODE(_initdata->trgmode);

	VEx->ERRINTEN = SET_ERRINTEN(cVERREN_INI);

	VEx->TMPREG0 = 0;											/* Temporary register clear */
	VEx->TMPREG1 = 0;
	VEx->TMPREG2 = 0;
	VEx->TMPREG3 = 0;
	VEx->TMPREG4 = 0;
	VEx->TMPREG5 = 0;

	VEx->TADC = (uint16_t)(cTADC * cIMCLK_FRQ * 1e6);			/* for 1shunt shift pwm.  (AD convert time) */

	/*********************** VE Register for Channel 1 **************************/
	VEx->MODE = VEMODE_STOP;

	if (_initdata->shunt == c3shunt)
	{
		VEx->FMODE = cFMODE_INI_3SHUNT;
	}
	else
	{
		VEx->FMODE = cFMODE_INI_1SHUNT;
	}

	VEx->TPWM = _initdata->tpwm;

	VEx->OMEGA = 0;
	VEx->THETA = 0;

	VEx->COS = 0;
	VEx->SIN = 0;
	VEx->COSM = 0;
	VEx->SINM = 0;

	VEx->IDREF = 0;
	VEx->IQREF = 0;

	VEx->VD = 0;
	VEx->VQ = 0;

	VEx->CIDKI = _initdata->idki;
	VEx->CIDKP = _initdata->idkp;
	VEx->CIDKG = _initdata->idkg;
	VEx->CIQKI = _initdata->iqki;
	VEx->CIQKP = _initdata->iqkp;
	VEx->CIQKG = _initdata->iqkg;

	VEx->VDIH = 0;
	VEx->VDILH = 0;
	VEx->VQIH = 0;
	VEx->VQILH = 0;

	VEx->MCTLF = 0x00;

	VEx->FPWMCHG = 0x7fff;										/* Hard control off */

	VEx->MINPLS = 0;

	VEx->SECTOR = 0;
	VEx->SECTORM = 0;

	VEx->IAO = _initdata->zerooffset;
	VEx->IBO = _initdata->zerooffset;
	VEx->ICO = _initdata->zerooffset;

	VEx->IAADC = 0;
	VEx->IBADC = 0;
	VEx->ICADC = 0;

	VEx->VDC = 0;
	VEx->ID = 0;
	VEx->IQ = 0;

	VEx->CMPU = 0;
	VEx->CMPV = 0;
	VEx->CMPW = 0;

	VEx->OUTCR = 0x0000;										/* Output UP/DOWN = OFF/OFF */

	VEx->TRGCRC = 0;

	VEx->TRGCMP0 = 0;
	VEx->TRGCMP1 = 0;

	VEx->TRGSEL = 0;

	VEx->EMGRS = 0;

	VEx->CPURUNTRG = VE_START;

}

/**
 *********************************************************************************************
 * @brief		ADC Initial set
 *
 * @param		TSB_AD_TypeDef* const	ADx			Select the ADC channel.
 * @param		AD_InitTypeDef* const	_initdata	The structure containing basic ADC configuration
 *
 * @return		none
 *********************************************************************************************
 */
void IP_ADC_init(TSB_AD_TypeDef* const ADx, AD_InitTypeDef* const _initdata)
{
	uint32_t data32;

	/***** PMD TRG Start mode set *****/

	CG_SetFcSupplyPeriph(CG_FC_PERIPH_DNFCK, ENABLE);
	CG_SetAdcClkSupply(ENABLE);
	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTD, ENABLE);
	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTE, ENABLE);
	CG_SetFsysSupplyPeriphB(CG_FC_PERIPH_ADC0, ENABLE);
	CG_SetFsysSupplyPeriphB(CG_FC_PERIPH_ADC1, ENABLE);

	if (_initdata->shunt == c3shunt)
	{
		/* ADx Program set */
		if (_initdata->pmd_ch == cPMD0)
		{
			ADx->PSEL0  = TRG_ENABLE(PMD_PROG0);		/* PMD0 TRG0:BProg0 at Sector0	3-shunt */
			ADx->PSEL1  = TRG_ENABLE(PMD_PROG1);		/* PMD0 TRG1:BProg1 at Sector1	3-shunt */
			ADx->PSEL2  = TRG_ENABLE(PMD_PROG2);		/* PMD0 TRG2:BProg2 at Sector2	3-shunt */
			ADx->PSEL3  = TRG_ENABLE(PMD_PROG3);		/* PMD0 TRG3:BProg3 at Sector3	3-shunt */
			ADx->PSEL4  = TRG_ENABLE(PMD_PROG4);		/* PMD0 TRG4:BProg4 at Sector4	3-shunt */
			ADx->PSEL5  = TRG_ENABLE(PMD_PROG5);		/* PMD0 TRG5:BProg5 at Sector5	3-shunt */
		}
		else if (_initdata->pmd_ch == cPMD1)
		{
			ADx->PSEL6  = TRG_ENABLE(PMD_PROG0);		/* PMD1 TRG0:BProg0 at Sector0	3-shunt */
			ADx->PSEL7  = TRG_ENABLE(PMD_PROG1);		/* PMD1 TRG1:BProg1 at Sector1	3-shunt */
			ADx->PSEL8  = TRG_ENABLE(PMD_PROG2);		/* PMD1 TRG2:BProg2 at Sector2	3-shunt */
			ADx->PSEL9  = TRG_ENABLE(PMD_PROG3);		/* PMD1 TRG3:BProg3 at Sector3	3-shunt */
			ADx->PSEL10 = TRG_ENABLE(PMD_PROG4);		/* PMD1 TRG4:BProg4 at Sector4	3-shunt */
			ADx->PSEL11 = TRG_ENABLE(PMD_PROG5);		/* PMD1 TRG5:BProg5 at Sector5	3-shunt */
		}
		
		if (_initdata->pmd_ch == cPMD0)
		{
			/* Getting AD convert registers address. */
			*_initdata->p_adreg0 = (uint32_t *)&ADx->REG0 + cPMD0_ADREG_OFFSET;
			*_initdata->p_adreg1 = (uint32_t *)&ADx->REG1 + cPMD0_ADREG_OFFSET;
			*_initdata->p_adreg2 = (uint32_t *)&ADx->REG2 + cPMD0_ADREG_OFFSET;
			*_initdata->p_adreg3 = (uint32_t *)&ADx->REG3 + cPMD0_ADREG_OFFSET;
			/* Setting AD convert registers to PRG0-5. */
			ADx->PREGS0 = cPMD0_PREGS_3SHUNT;
		}
		else if (_initdata->pmd_ch == cPMD1)
		{
			/* Getting AD convert registers address. */
			*_initdata->p_adreg0 = (uint32_t *)&ADx->REG0 + cPMD1_ADREG_OFFSET;
			*_initdata->p_adreg1 = (uint32_t *)&ADx->REG1 + cPMD1_ADREG_OFFSET;
			*_initdata->p_adreg2 = (uint32_t *)&ADx->REG2 + cPMD1_ADREG_OFFSET;
			*_initdata->p_adreg3 = (uint32_t *)&ADx->REG3 + cPMD1_ADREG_OFFSET;
			/* Setting AD convert registers to PRG0-5. */
			ADx->PREGS0 = cPMD1_PREGS_3SHUNT;
		}

		if (_initdata->pints == cPINTS_A)
		{
			ADx->PINTS0 = PMD_INTADPDA;					/* End of all prog generate INTADA */
			ADx->PINTS1 = PMD_INTADPDA;
			ADx->PINTS2 = PMD_INTADPDA;
			ADx->PINTS3 = PMD_INTADPDA;
			ADx->PINTS4 = PMD_INTADPDA;
			ADx->PINTS5 = PMD_INTADPDA;
			ADx->PINTS6 = PMD_INTNONE;
			ADx->PINTS7 = PMD_INTNONE;
			ADx->PINTS8 = PMD_INTNONE;
		}
		else if (_initdata->pints == cPINTS_B)
		{
			ADx->PINTS0 = PMD_INTADPDB;					/* End of all prog generate INTADB */
			ADx->PINTS1 = PMD_INTADPDB;
			ADx->PINTS2 = PMD_INTADPDB;
			ADx->PINTS3 = PMD_INTADPDB;
			ADx->PINTS4 = PMD_INTADPDB;
			ADx->PINTS5 = PMD_INTADPDB;
			ADx->PINTS6 = PMD_INTNONE;
			ADx->PINTS7 = PMD_INTNONE;
			ADx->PINTS8 = PMD_INTNONE;
		}
		else if (_initdata->pints == cPINTS_C)
		{
			ADx->PINTS0 = PMD_INTADPDC;					/* End of all prog generate INTADC */
			ADx->PINTS1 = PMD_INTADPDC;
			ADx->PINTS2 = PMD_INTADPDC;
			ADx->PINTS3 = PMD_INTADPDC;
			ADx->PINTS4 = PMD_INTADPDC;
			ADx->PINTS5 = PMD_INTADPDC;
			ADx->PINTS6 = PMD_INTNONE;
			ADx->PINTS7 = PMD_INTNONE;
			ADx->PINTS8 = PMD_INTNONE;
		}
		else
		{
			ADx->PINTS0 = PMD_INTADPDD;					/* End of all prog generate INTADD */
			ADx->PINTS1 = PMD_INTADPDD;
			ADx->PINTS2 = PMD_INTADPDD;
			ADx->PINTS3 = PMD_INTADPDD;
			ADx->PINTS4 = PMD_INTADPDD;
			ADx->PINTS5 = PMD_INTADPDD;
			ADx->PINTS6 = PMD_INTNONE;
			ADx->PINTS7 = PMD_INTNONE;
			ADx->PINTS8 = PMD_INTNONE;
		}

		/* ADx->PSET5,PSET0	REG0:V, REG1:W, REG2:U, REG3:Vdc */
		data32 = (((0x80 | (VE_PHASE_V    << 5U) | _initdata->ivch)) \
		        | ((0x80 | (VE_PHASE_W    << 5U) | _initdata->iwch)  << 8U) \
		        | ((0x80 | (VE_PHASE_U    << 5U) | _initdata->iuch)  << 16U) \
		        | ((0x80 | (VE_PHASE_NONE << 5U) | _initdata->vdcch) << 24U));
		ADx->PSET5 = data32;
		ADx->PSET0 = data32;

		/* ADx->PSET1,PSET2	REG0:W, REG1:U, REG2:V, REG3:Vdc */
		data32 = (((0x80 | (VE_PHASE_W    << 5U) | _initdata->iwch)) \
		        | ((0x80 | (VE_PHASE_U    << 5U) | _initdata->iuch)  << 8U) \
		        | ((0x80 | (VE_PHASE_V    << 5U) | _initdata->ivch)  << 16U) \
		        | ((0x80 | (VE_PHASE_NONE << 5U) | _initdata->vdcch) << 24U));
		ADx->PSET1 = data32;
		ADx->PSET2 = data32;

		/* ADx->PSET3,PSET4	REG0:U, REG1:V, REG2:W, REG3:Vdc */
		data32 = (((0x80 | (VE_PHASE_U    << 5U) | _initdata->iuch)) \
		        | ((0x80 | (VE_PHASE_V    << 5U) | _initdata->ivch)  << 8U) \
		        | ((0x80 | (VE_PHASE_W    << 5U) | _initdata->iwch)  << 16U) \
		        | ((0x80 | (VE_PHASE_NONE << 5U) | _initdata->vdcch) << 24U));
		ADx->PSET3	= data32;
		ADx->PSET4	= data32;

		ADx->PSET6	= 0;
		ADx->PSET7	= 0;
		ADx->PSET8	= 0;

	}
	else if (_initdata->shunt == c1shunt)
	{
		/* ADx Program set */
		if (_initdata->pmd_ch == cPMD0)
		{
			ADx->PSEL0  = TRG_ENABLE(PMD_PROG0);		/* PMD0 TRG0:BProg0 1-shunt */
			ADx->PSEL1  = TRG_ENABLE(PMD_PROG1);		/* PMD0 TRG1:BProg1 1-shunt */
			ADx->PSEL2  = TRG_ENABLE(PMD_PROG2);		/* PMD0 TRG2:BProg2 */
			ADx->PSEL3  = TRG_DISABLE(PMD_PROG0);		/* PMD0 TRG3:BProg3 */
			ADx->PSEL4  = TRG_DISABLE(PMD_PROG0);		/* PMD0 TRG4:BProg4 */
			ADx->PSEL5  = TRG_DISABLE(PMD_PROG0);		/* PMD0 TRG5:BProg5 */
		}
		else if (_initdata->pmd_ch == cPMD1)
		{
			ADx->PSEL6  = TRG_ENABLE(PMD_PROG0);		/* PMD1 TRG0:BProg0 1-shunt */
			ADx->PSEL7  = TRG_ENABLE(PMD_PROG1);		/* PMD1 TRG1:BProg1 1-shunt */
			ADx->PSEL8  = TRG_ENABLE(PMD_PROG2);		/* PMD1 TRG2:BProg2 */
			ADx->PSEL9  = TRG_DISABLE(PMD_PROG0);		/* PMD1 TRG3:BProg3 */
			ADx->PSEL10 = TRG_DISABLE(PMD_PROG0);		/* PMD1 TRG4:BProg4 */
			ADx->PSEL11 = TRG_DISABLE(PMD_PROG0);		/* PMD1 TRG5:BProg5 */
		}

		if (_initdata->pmd_ch == cPMD0)
		{
			/* Getting AD convert registers address. */
			*_initdata->p_adreg0 = (uint32_t *)&ADx->REG0 + cPMD0_ADREG_OFFSET;
			*_initdata->p_adreg1 = (uint32_t *)&ADx->REG1 + cPMD0_ADREG_OFFSET;
			*_initdata->p_adreg2 = (uint32_t *)&ADx->REG2 + cPMD0_ADREG_OFFSET;
			*_initdata->p_adreg3 = (uint32_t *)&ADx->REG3 + cPMD0_ADREG_OFFSET;
			/* Setting AD convert registers to PRG0-2. */
			ADx->PREGS0 = cPMD0_PREGS_1SHUNT;
		}
		else if (_initdata->pmd_ch == cPMD1)
		{
			/* Getting AD convert registers address. */
			*_initdata->p_adreg0 = (uint32_t *)&ADx->REG0 + cPMD1_ADREG_OFFSET;
			*_initdata->p_adreg1 = (uint32_t *)&ADx->REG1 + cPMD1_ADREG_OFFSET;
			*_initdata->p_adreg2 = (uint32_t *)&ADx->REG2 + cPMD1_ADREG_OFFSET;
			*_initdata->p_adreg3 = (uint32_t *)&ADx->REG3 + cPMD1_ADREG_OFFSET;
			/* Setting AD convert registers to PRG0-2. */
			ADx->PREGS0 = cPMD1_PREGS_1SHUNT;
		}

		if (_initdata->pints == cPINTS_A)
		{
			ADx->PINTS0 = PMD_INTNONE; 					/* End of all prog generate INTADA */
			ADx->PINTS1 = PMD_INTNONE;
			ADx->PINTS2 = PMD_INTADPDA;
			ADx->PINTS3 = PMD_INTNONE;
			ADx->PINTS4 = PMD_INTNONE;
			ADx->PINTS5 = PMD_INTNONE;
			ADx->PINTS6 = PMD_INTNONE;
			ADx->PINTS7 = PMD_INTNONE;
			ADx->PINTS8 = PMD_INTNONE;
		}
		else if (_initdata->pints == cPINTS_B)
		{
			ADx->PINTS0 = PMD_INTNONE; 					/* End of all prog generate INTADB */
			ADx->PINTS1 = PMD_INTNONE;
			ADx->PINTS2 = PMD_INTADPDB;
			ADx->PINTS3 = PMD_INTNONE;
			ADx->PINTS4 = PMD_INTNONE;
			ADx->PINTS5 = PMD_INTNONE;
			ADx->PINTS6 = PMD_INTNONE;
			ADx->PINTS7 = PMD_INTNONE;
			ADx->PINTS8 = PMD_INTNONE;
		}
		else if (_initdata->pints == cPINTS_C)
		{
			ADx->PINTS0 = PMD_INTNONE; 					/* End of all prog generate INTADC */
			ADx->PINTS1 = PMD_INTNONE;
			ADx->PINTS2 = PMD_INTADPDC;
			ADx->PINTS3 = PMD_INTNONE;
			ADx->PINTS4 = PMD_INTNONE;
			ADx->PINTS5 = PMD_INTNONE;
			ADx->PINTS6 = PMD_INTNONE;
			ADx->PINTS7 = PMD_INTNONE;
			ADx->PINTS8 = PMD_INTNONE;
		}
		else
		{
			ADx->PINTS0 = PMD_INTNONE; 					/* End of all prog generate INTADD */
			ADx->PINTS1 = PMD_INTNONE;
			ADx->PINTS2 = PMD_INTADPDD;
			ADx->PINTS3 = PMD_INTNONE;
			ADx->PINTS4 = PMD_INTNONE;
			ADx->PINTS5 = PMD_INTNONE;
			ADx->PINTS6 = PMD_INTNONE;
			ADx->PINTS7 = PMD_INTNONE;
			ADx->PINTS8 = PMD_INTNONE;
		}
		

		/*ADx->PSET0	REG0:Idc----, REG1:-------, REG2:-------, REG3:------- */
		data32 = (0x80 | _initdata->idcch);
		ADx->PSET0 = data32;

		/*ADx->PSET1	REG0:-------, REG1:Idc----, REG2:-------, REG3:------- */
		data32 = (((0x80 | _initdata->idcch) << 8U));
		ADx->PSET1 = data32;

		/*ADx->PSET2	REG0:-------, REG1:-------, REG2:-------, REG3:Vdc---- */
		data32 = ((0x80 | _initdata->vdcch) << 24U);
		ADx->PSET2 = data32;

		ADx->PSET3 = 0;
		ADx->PSET4 = 0;
		ADx->PSET5 = 0;
		ADx->PSET6 = 0;
		ADx->PSET7 = 0;
		ADx->PSET8 = 0;
	}

}


/**
 *********************************************************************************************
 * @brief		AMP IP Initialize
 *
 * @return		none
 *********************************************************************************************
 */
void IP_AMP_init(void)
{

#if defined(__AMP)
	CG_SetFsysSupplyPeriphB(CG_FC_PERIPH_AMP, ENABLE);
	/* Setting internal amp */
	TSB_AMP->CTLA = AMP_ENABLE(AMPGAIN_FOLD_6_0);
	TSB_AMP->CTLB = AMP_ENABLE(AMPGAIN_FOLD_6_0);
	TSB_AMP->CTLC = AMP_ENABLE(AMPGAIN_FOLD_6_0);
#endif

}

/**
 ********************************************************************************************
 * @brief		Initialize PLL Select Register for ADCLK.
 *
 * @return		none
 ********************************************************************************************
 */
void IP_PLL_ADCLK_init(void)
{
	TSB_CG_TypeDef* const CGx = TSB_CG;

	/* PLL for AD oscillating ? */
	if ((CGx->PLL1SEL & ~(1 << B_PLL1ON)) != 0)
	{
		CGx->PLL1SEL &= ~(1 << B_PLL1SEL);				/* ADCLK : fc */
		CGx->PLL1SEL &= ~(1 << B_PLL1ON);				/* PLL : Stop */
	}

	/* Set multiplication value of the PLL. */
	CGx->PLL1SEL = (cPLL1SET << 8U);
	/* Wait */
	CGx->WUPHCR = cWUPHCR_INIT_TIME;
	CGx->WUPHCR = (cWUPHCR_INIT_TIME | WUON_START);
	while (TSB_CG_WUPHCR_WUEF == 1){ /* do nothing */ };

	CGx->PLL1SEL |= (1 << B_PLL1ON);					/* PLL : Oscillation */
	/* Wait */
	CGx->WUPHCR = cWUPHCR_LOCKUP_TIME;
	CGx->WUPHCR = (cWUPHCR_LOCKUP_TIME | WUON_START);
	while (TSB_CG_WUPHCR_WUEF == 1){ /* do nothing */ };

	CGx->PLL1SEL |= (1 << B_PLL1SEL);					/* ADCLK : fPLL*/

}


/*********************************** END OF FILE ******************************/
