/**
 ****************************************************************************
 * @file	usercon.c
 * @brief	User Control
 * @version	V1.0.2
 * 
 * DO NOT USE THIS SOFTWARE WITHOUT THE SOFTWARE LICENSE AGREEMENT.
 * 
 * Copyright(C) Toshiba Electronic Device Solutions Corporation 2019
 *****************************************************************************
 */
#include <stdlib.h>
#include <stdio.h>
#include "ipdefine.h"

#include "mcuip_drv.h"
#include "D_Para.h"

#include "ipdrv_adc.h"
#include "ipdrv_cg.h"

#include "dac_drv.h"

#define DEFINE_APP
#include "usercon.h"
#undef	DEFINE_APP


/*===================================================================*
	  Macro Definition
 *===================================================================*/
#define	__VR_SPEED			(1) 	/* 0: Not update, 1: Update target_spd  */

/*===================================================================*
	  Local Constant Data Definition
 *===================================================================*/
/* Key settting */
#define S_SW1				TSB_PL_DATA_PL0					/* S_SW1 */
#define S_SW2				TSB_PL_DATA_PL1					/* S_SW2 */
#define S_SW3				TSB_PL_DATA_PL2					/* S_SW3 */

#define	P_SW1				TSB_PE_DATA_PE2 				/* USW1 */
#define	P_SW2				TSB_PE_DATA_PE3 				/* USW2 */

#define	cKEY_CHATA_CNT		(20)							/* [cnt] chattering counter for KEY SW */

/* Soft ADC Setting */
#define	cADUNIT_USR			TSB_ADA							/* User ad data ADCUnit */

#define	cADCH_VR			ADC_AIN14						/* ADC Channel for VR */
#define	cADREG_VR			ADC_REG5						/* Result register No for VR */
#define cADCH_TEMP			ADC_AIN10						/* ADC Channel for TEMP */
#define cADREG_TEMP			ADC_REG6						/* Result register No for TEMP */

#define	cADAVECNT			(10)							/* ADC average count */
#define	cPUSHSW_CHATA_CNT	(5) 							/* [cnt] Chattering counter for Push SW */

/* UART Setting */
#define UART_BRD_INIT		((uint32_t)0x00A7002B)
#define UART_CLK_INIT		((uint32_t)0x00000000)
#define UART_CR0_INIT		((uint32_t)0x00000001)
#define UART_CR1_INIT		((uint32_t)0x00000040)

#define UART_TXE_ENABLE		((uint32_t)0x00000002)
#define UART_TXE_DISABLE	((uint32_t)0x00000000)

#define UART_ERR_CHECK		((uint32_t)0x0000001F)
#define UART_ERR_CLR		((uint32_t)0x0000001F)

#define UART_TX_FLAG_CLR	((uint32_t)0x00004000)

#define UART_SEND_WAIT		(1000)

#define cSEND_DATA_NUM		(50)

/* User Setting */
#define cROTATION_CW		(0)
#define cROTATION_CCW		(1)

#define cSPDCTL_PUSH		(0)
#define cSPDCTL_VR			(1)


/* NBD Control Setting */
#define	cNBD_ROTATION		(cROTATION_CW)		/* Rotation 		CW: cROTATION_CW  CCW: cROTATION_CCW */
#define	cNBD_SPDCTL			(cSPDCTL_VR)		/* Speed Control	VR: cSPDCTL_VR  PUSH SW: cSPDCTL_PUSH */
#define	cNBD_PHASE			(c2PHASE)			/* Phase			2Phase: c2PHASE  3Phase: c3PHASE */


/*===================================================================*
	  Typedef Definition
 *===================================================================*/
typedef struct
{
	uint8_t		select;										/* Dac output select */
	uint8_t		motch;										/* Dac motor ch select */
	uint8_t		datsft0;									/* Dac data shift for data0 */
	uint8_t		datsft1;									/* Dac data shift for data1 */
	uint8_t		datsft2;									/* Dac data shift for data2 */
	uint8_t		datsft3;									/* Dac data shift for data3 */
} dac_t;


#if defined(__CONTROL_MOTOR_CH1)
extern	int32_t	target_spd1;
#endif /* __CONTROL_MOTOR_CH1 */

#if defined(__CONTROL_MOTOR_CH0)
extern	int32_t	target_spd0;
#endif /* __CONTROL_MOTOR_CH0 */


/*===================================================================*
	  Local Variable Definition
 *===================================================================*/
#if !defined(__USE_NBD)
static ssw_t		keyportbuf_old; 						/* Previous Key port data */
static uint8_t		key_chatacnt;							/* Chattering counter for Key */
#endif
static psw_t		p_keyportbuf_old;						/* Previous Push Key port data */
static uint8_t		p_key_chatacnt;							/* Chattering counter for Push Key */
static uint8_t		sw_chg_stop;

static addat_t		ad_vr;									/* AD value of volume register */
static uint8_t		vr_get;									/* Volume register value get */

static uint8_t		speed_ctr_mode;							/* Speed control switch setting */
static uint8_t		rote_dir;								/* Rotation direction setting */

static int32_t		push_target_speed;						/* Target speed by Push SW */
static uint8_t		push_sw_chg;							/* Input change for Push SW */
static uint8_t		sw_get;									/* TEMP value get */

static addat_t		ad_temp;								/* AD value of TEMP */
static uint8_t		temp_get;								/* sw value get */

static uint8_t		txsize;									/* Size of send data */
static uint8_t		txpnt;									/* Pointer of send data  */
static uint8_t		sendbuffer[cSEND_DATA_NUM];				/* Send data buffer */
static uint8_t		send_step;
static uint16_t		send_wait_time;

static uint8_t		start_motor;

				 /* No, MotorCh, sft0, sft1, sft2, sft3 */
dac_t 		dac = {  0,       0,    0,    0,    0,    0 };	/* Dac setting. */



/*===================================================================*
	  Local Proto Type Definition
 *===================================================================*/
static void init_Uikey(void);
static void Uikeyscan(void);
static void init_soft_adc(void);
static void soft_adc_getdata(void);
static void soft_adc_avecal(addat_t* const _ad_dat);
static void vr2speed(void);
static void pushspeed(void);
#if !defined(__USE_NBD)
static void key_setting(void);
#endif
static void	temp_data_set(void);
static int16_t temp_check(uint16_t	temp_ad);
static void init_led(void);
static void led_out(void);
static void init_uart(void);
static void send_data_set(void);
static void number_send_data_set(uint32_t _num, uint8_t _digit, uint8_t _decimal);

/**
 *********************************************************************************************
 * @brief		init_user_control
 *
 * @return		none
 *********************************************************************************************
 */
void	init_user_control(void)
{
	init_Uikey();										/* Initial setting of key input */
	init_soft_adc();									/* Initial setting of soft adc */
	init_led();											/* Initial setting of LED output */
	init_uart();										/* Initial setting of UART */
}

/**
 *********************************************************************************************
 * @brief		user_control
 *
 * @return		none
 *********************************************************************************************
 */
void	user_control(void)
{
	/***** Get AD Data *****/
	soft_adc_getdata();

	/***** key input *****/
	Uikeyscan();
#if !defined(__USE_NBD)
	key_setting();
#endif

	/***** ad volume input *****/
	if(speed_ctr_mode == cSPDCTL_VR){
		vr2speed();
	}
	else
	{
		pushspeed();
	}

	/***** TEMP input *****/
	temp_data_set();

	/***** LED output *****/
	led_out();

	/***** UART send *****/
	if((temp_get == 1) && (sw_get == 1)){
		send_data_set();
	}

}

/**
 *********************************************************************************************
 * @brief		vr2speed
 *
 * @return		none
 *********************************************************************************************
 */
#define		cAD_MIN		0x10
#define		cAD_MAX		0xF0

static void	vr2speed(void)
{
	uint32_t ad_vr_temp;
	int32_t speed_temp;

	ad_vr_temp = ad_vr.avedat >> 4U;

#if defined(__CONTROL_MOTOR_CH1)
	if (ad_vr_temp < cAD_MIN)
	{
		speed_temp = 0;
	}
	else if (ad_vr_temp < cAD_MAX)
	{
		speed_temp = (cSPEED_ACT_MAX_CH1 - cSPEED_ACT_MIN_CH1) * (ad_vr_temp - cAD_MIN) / (cAD_MAX - cAD_MIN) + cSPEED_ACT_MIN_CH1;
	}
	else
	{
		speed_temp = cSPEED_ACT_MAX_CH1;
	}

	/* check 1st vr value = 0	*/
	if( 0 == vr_get ){
		if( 0 == ad_vr.avecnt ){
			if( 0 != speed_temp ){
				speed_temp = 0;
			}
			else{
				vr_get = 1;
			}
		}
		else{
			speed_temp = 0;
		}
	}
	else{
		/* do nothing	*/
	}

	if(sw_chg_stop == 1)
	{
		if(speed_temp == 0){
			sw_chg_stop = 0;
		}
		else
		{
			speed_temp = 0;
		}
	}
	if(rote_dir == cROTATION_CW)
	{
		target_spd1 = speed_temp;
	}
	else
	{
		target_spd1 = -speed_temp;
	}
#endif

#if defined(__CONTROL_MOTOR_CH0)
	if (ad_vr_temp < cAD_MIN)
	{
		speed_temp = 0;
	}
	else if (ad_vr_temp < cAD_MAX)
	{
		speed_temp = (cSPEED_ACT_MAX_CH0 - cSPEED_ACT_MIN_CH0) * (ad_vr_temp - cAD_MIN) / (cAD_MAX - cAD_MIN) + cSPEED_ACT_MIN_CH0;
	}
	else
	{
		speed_temp = cSPEED_ACT_MAX_CH0;
	}

	/* check 1st vr value = 0	*/
	if( 0 == vr_get ){
		if( 0 == ad_vr.avecnt ){
			if( 0 != speed_temp ){
				speed_temp = 0;
			}
			else{
				vr_get = 1;
			}
		}
		else{
			speed_temp = 0;
		}
	}
	else{
		/* do nothing	*/
	}

	if(sw_chg_stop == 1)
	{
		if(speed_temp == 0){
			sw_chg_stop = 0;
		}
		else
		{
			speed_temp = 0;
		}
	}

	if(rote_dir == cROTATION_CW)
	{
		target_spd0 = speed_temp;
	}
	else
	{
		target_spd0 = -speed_temp;
	}
#endif

	if(speed_temp != 0){
		start_motor = 1;
	}

}



/**
 *********************************************************************************************
 * @brief		pushspeed
 *
 * @return		none
 *********************************************************************************************
 */
static void pushspeed(void)
{
#if defined(__CONTROL_MOTOR_CH1)
	vector_t* const _motor = &Motor_ch1;

	if(push_sw_chg == 1){
		push_sw_chg = 0;
		if(_motor->stage.main == cEmergency){
			if((push_keydata.p_sw1 == 1) || (push_keydata.p_sw2 == 1)){
				push_target_speed = 0;
			}
		}
		else{
			if(sw_chg_stop == 1){
				push_target_speed = 0;
				sw_chg_stop = 0;
			}
			else{
				if((push_keydata.p_sw1 == 1) && (push_target_speed < cSPEED_ACT_MAX_CH1)){
					push_target_speed += 10;
				}
				else if((push_keydata.p_sw2 == 1) && (push_target_speed > 0)){
					push_target_speed -= 10;
				}
				else{
				}
			}
		}
	}

	if(sw_chg_stop == 1){
		target_spd1 = 0;
	}
	else if(rote_dir == cROTATION_CW){
		target_spd1 = push_target_speed;
	}
	else{
		target_spd1 = -push_target_speed;
	}

#endif

#if defined(__CONTROL_MOTOR_CH0)
	vector_t* const _motor = &Motor_ch0;

	if(push_sw_chg == 1){
		push_sw_chg = 0;
		if(_motor->stage.main == cEmergency){
			if((push_keydata.p_sw1 == 1) || (push_keydata.p_sw2 == 1)){
				push_target_speed = 0;
			}
		}
		else{
			if(sw_chg_stop == 1){
				push_target_speed = 0;
				sw_chg_stop = 0;
			}
			else{
				if((push_keydata.p_sw1 == 1) && (push_target_speed < cSPEED_ACT_MAX_CH0)){
					push_target_speed += 10;
				}
				else if((push_keydata.p_sw2 == 1) && (push_target_speed > 0)){
					push_target_speed -= 10;
				}
				else{
				}
			}
		}
	}

	if(sw_chg_stop == 1){
		target_spd0 = 0;
	}
	else if(rote_dir == cROTATION_CW){
		target_spd0 = push_target_speed;
	}
	else{
		target_spd0 = -push_target_speed;
	}

#endif


	if(push_target_speed != 0){
		start_motor = 1;
	}

}



#if !defined(__USE_NBD)
/**
 *********************************************************************************************
 * @brief		key_setting
 *
 * @return		none
 *********************************************************************************************
 */
static void key_setting(void)
{
#if defined(__CONTROL_MOTOR_CH1)
	vector_t* const _motor = &Motor_ch1;

	/* Rotation direction setting */
	if(keydata.s_sw1 == 1){
		rote_dir = cROTATION_CW;
	}
	else{
		rote_dir = cROTATION_CCW;
	}

	/* Modulation type setting */
	if(keydata.s_sw2 == 1){
		_motor->usr.com_user.modul = c2PHASE;
	}
	else
	{
		_motor->usr.com_user.modul = c3PHASE;
	}

	/* Rotational speed control selection */
	if(keydata.s_sw3 == 1){
		speed_ctr_mode = cSPDCTL_VR;
	}
	else{
		speed_ctr_mode = cSPDCTL_PUSH;
	}
#endif

#if defined(__CONTROL_MOTOR_CH0)
	vector_t* const _motor = &Motor_ch0;

	/* Rotation direction setting */
	if(keydata.s_sw1 == 1){
		rote_dir = cROTATION_CW;
	}
	else{
		rote_dir = cROTATION_CCW;
	}

	/* Modulation type setting */
	if(keydata.s_sw2 == 1){
		_motor->usr.com_user.modul = c2PHASE;
	}
	else
	{
		_motor->usr.com_user.modul = c3PHASE;
	}

	/* Rotational speed control selection */
	if(keydata.s_sw3 == 1){
		speed_ctr_mode = cSPDCTL_VR;
	}
	else{
		speed_ctr_mode = cSPDCTL_PUSH;
	}
#endif
}
#endif



/**
 *********************************************************************************************
 * @brief		Initialize key.
 *
 * @return		none
 *********************************************************************************************
 */
static void init_Uikey(void)
{
#if defined(__USE_NBD)
	sw_get = 1;
	rote_dir = cNBD_ROTATION;
	speed_ctr_mode = cNBD_SPDCTL;
	Motor_ch0.usr.com_user.modul = cNBD_PHASE;
#else
	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTL, ENABLE);
	/* S_SW1 */
	TSB_PL_IE_PL0IE = SET;
	/* S_SW2 */
	TSB_PL_IE_PL1IE = SET;
	/* S_SW3 */
	TSB_PL_IE_PL2IE = SET;

	*((uint8_t*)&keyportbuf_old) = 0xff;
#endif /* __USE_NBD */

	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTE, ENABLE);
	/* USW1 */
	TSB_PE_IE_PE2IE = SET;
	/* USW2 */
	TSB_PE_IE_PE3IE = SET;

	start_motor = 0;
}




/**
 *********************************************************************************************
 * @brief		keyscan
 *
 * @return		none
 *********************************************************************************************
 */
static void Uikeyscan(void)
{
#if !defined(__USE_NBD)
	ssw_t	portbuf;
#endif
	psw_t	portbuf_2;

#if !defined(__USE_NBD)
	*((uint8_t*)&portbuf) = 0;
	portbuf.s_sw1 = S_SW1;
	portbuf.s_sw2 = S_SW2;
	portbuf.s_sw3 = S_SW3;
#endif

	*((uint8_t*)&portbuf_2) = 0;
	portbuf_2.p_sw1 = P_SW1;
	portbuf_2.p_sw2 = P_SW2;

#if !defined(__USE_NBD)
	/* TSW1-3 Check */
	if (*((uint8_t*)&portbuf) == *((uint8_t*)&keyportbuf_old))
	{
		if (key_chatacnt != 0)
		{
			if (--key_chatacnt == 0)
			{
				*((uint8_t*)&keydata) = (*((uint8_t*)&portbuf));
				sw_chg_stop = 1;
				sw_get = 1;
			}
		}
	}
	else
	{
		*((uint8_t*)&keyportbuf_old) = *((uint8_t*)&portbuf);
		key_chatacnt = cKEY_CHATA_CNT;
	}
#endif

	/* Push SW1-2 Check */
	if (*((uint8_t*)&portbuf_2) == *((uint8_t*)&p_keyportbuf_old))
	{
		if (p_key_chatacnt != 0)
		{
			if (--p_key_chatacnt == 0)
			{
				if((portbuf_2.p_sw1 == 1) || (portbuf_2.p_sw2 == 1)){
					*((uint8_t*)&push_keydata) = ~(*((uint8_t*)&portbuf_2));
					push_sw_chg = 1;
				}
			}
		}
	}
	else
	{
		*((uint8_t*)&p_keyportbuf_old) = *((uint8_t*)&portbuf_2);
		p_key_chatacnt = cKEY_CHATA_CNT;
	}
}

/**
 *********************************************************************************************
 * @brief		Initialize softconvert adc.
 *
 * @return		none
 *********************************************************************************************
 */
static void init_soft_adc(void)
{
	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTD, ENABLE);
	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTE, ENABLE);

	/* VR1(ADC_AIN14) PE4 */
	/* TEMP(AIN10) PE0 */
	ADC_SetSWTrg(cADUNIT_USR, cADREG_VR, TRG_ENABLE(cADCH_VR));
	ADC_SetSWTrg(cADUNIT_USR, cADREG_TEMP, TRG_ENABLE(cADCH_TEMP));

	ADC_Start(cADUNIT_USR, ADC_TRG_SINGLE); 							/* adc start */
}


/**
 *********************************************************************************************
 * @brief		soft_adc_getdata
 *
 * @return		none
 *********************************************************************************************
 */
static void soft_adc_getdata(void)
{
	if (ADC_GetConvertState(cADUNIT_USR, ADC_TRG_SINGLE) == DONE)		/* finish adc? */
	{
		ad_vr.rawdat = ADC_GetConvertResult(cADUNIT_USR, cADREG_VR);
		ad_temp.rawdat = ADC_GetConvertResult(cADUNIT_USR, cADREG_TEMP);
		soft_adc_avecal(&ad_vr);
		soft_adc_avecal(&ad_temp);

	}
	ADC_Start(cADUNIT_USR, ADC_TRG_SINGLE); 							/* adc start */
}


/**
 *********************************************************************************************
 * @brief		soft_adc_avecal
 *
 * @param		addat_t* const	_ad_dat
 *
 * @return		none
 *********************************************************************************************
 */
static void soft_adc_avecal(addat_t* const _ad_dat)
{

	if (_ad_dat->avecnt == 0)
	{
		/* first? */
		_ad_dat->maxdat = _ad_dat->rawdat.Bit.ADResult;
		_ad_dat->mindat = _ad_dat->rawdat.Bit.ADResult;
	}
	else
	{
		if (_ad_dat->rawdat.Bit.ADResult > _ad_dat->maxdat)
		{
			_ad_dat->maxdat = _ad_dat->rawdat.Bit.ADResult;
		}
		else if (_ad_dat->rawdat.Bit.ADResult < _ad_dat->mindat)
		{
			_ad_dat->mindat = _ad_dat->rawdat.Bit.ADResult;
		}
	}

	_ad_dat->sumdat += _ad_dat->rawdat.Bit.ADResult;

	_ad_dat->avecnt++;
	if (_ad_dat->avecnt >= cADAVECNT)
	{
		_ad_dat->avecnt = 0;
		_ad_dat->sumdat -= _ad_dat->maxdat;
		_ad_dat->sumdat -= _ad_dat->mindat;
		_ad_dat->avedat = (_ad_dat->sumdat / (cADAVECNT - 2));
		_ad_dat->sumdat = 0;
	}

}



/**
 *********************************************************************************************
 * @brief		temp_data_set
 *
 * @return		adkeycode_e
 *********************************************************************************************
 */
static void temp_data_set(void)
{
	if(temp_get == 0)
	{
		if(ad_temp.avecnt == 0)
		{
			temp = temp_check(ad_temp.avedat);
			temp_get = 1;
		}
	}
}



/**
 ********************************************************************************************
 * @brief		Temperature calculation from AD.
 *
 * @param[in]	uint8_t	temp_ad	.
 *
 * @return		none
 ********************************************************************************************
 */
#define		cTEMPAD_MAX		(3656)
#define		cTEMPAD_MIN		(345)

#define		cTEMP_MAX		(1000)
#define		cTEMP_MIN		(-200)
#define		cTEMP_ERR		(2000)

typedef struct {
	int16_t		temp_value;
	uint16_t	ad_value;
} TYPE_TEMP;

TYPE_TEMP	t_temppara[] = {
		{	-200,	3656},
		{	-100,	3323},
		{	   0,	2958},
		{	 100,	2550},
		{	 200,	2146},
		{	 300,	1755},
		{	 400,	1419},
		{	 500,	1128},
		{	 600,	 896},
		{	 700,	 711},
		{	 800,	 565},
		{	 900,	 439},
		{	1000,	 345},
};

static int16_t temp_check(uint16_t temp_ad)
{
	int16_t	percent, slope, def, temp_data;
	uint8_t	i;

	temp_data = cTEMP_ERR;
	if(temp_ad < cTEMPAD_MIN){
		/* do nothing	*/
	}
	else if(temp_ad > cTEMPAD_MAX){
		/* do nothing	*/
	}
	else{
		for(i = 0; i < (sizeof(t_temppara)/sizeof(t_temppara[0])) ; i++){
			if(temp_ad > t_temppara[i].ad_value ){
				break;
			}
			else{
				/* do nothing	*/
			}
		}
		if( i < (sizeof(t_temppara)/sizeof(t_temppara[0])) ){
			/* do nothing	*/
		}
		else{
			i = (sizeof(t_temppara)/sizeof(t_temppara[0]));
		}

		/* get temp. value	*/
		slope = (int16_t)(t_temppara[i - 1].ad_value - t_temppara[i].ad_value);
		def = (int16_t)(t_temppara[i - 1].ad_value - temp_ad);
		percent = (int16_t)((t_temppara[i].temp_value - t_temppara[i - 1].temp_value) * def / slope);
		temp_data = (int16_t)(t_temppara[i - 1].temp_value + percent);
	}

	return(temp_data);
}




/**
 *********************************************************************************************
 * @brief		Initialize LED.
 *
 * @return		none
 *********************************************************************************************
 */
static void init_led(void)
{
	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTG, ENABLE);
	/* LED4 */
	TSB_PG_CR_PG3C = SET;
	/* LED5 */
	TSB_PG_CR_PG4C = SET;
	/* LED6 */
	TSB_PG_CR_PG5C = SET;
}



/**
 *********************************************************************************************
 * @brief		LED output.
 *
 * @return		none
 *********************************************************************************************
 */
static void led_out(void)
{
#if defined(__CONTROL_MOTOR_CH1)
	vector_t* const _motor = &Motor_ch1;

	/* LED4 */
	if((_motor->stage.main >= cInitposition) && (_motor->stage.main < cSteady_A)){
		TSB_PG_DATA_PG3 = 1;
	}
	else{
		TSB_PG_DATA_PG3 = 0;
	}

	/* LED6 */
	if(_motor->stage.main == cEmergency){
		TSB_PG_DATA_PG5 = 1;
	}
	else{
		TSB_PG_DATA_PG5 = 0;
	}
#endif
#if defined(__CONTROL_MOTOR_CH0)
	vector_t* const _motor = &Motor_ch0;

	/* LED4 */
	if((_motor->stage.main >= cInitposition) && (_motor->stage.main < cSteady_A)){
		TSB_PG_DATA_PG3 = 1;
	}
	else{
		TSB_PG_DATA_PG3 = 0;
	}

	/* LED6 */
	if(_motor->stage.main == cEmergency){
		TSB_PG_DATA_PG5 = 1;
	}
	else{
		TSB_PG_DATA_PG5 = 0;
	}
#endif
}



/**
 *********************************************************************************************
 * @brief		Initialize Test point
 *
 * @return		none
 *********************************************************************************************
 */
void init_TP(void)
{
	/* port setting */
	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTA, ENABLE);
	TSB_PA->CR |= ((1 << 0U) | (1 << 1U));	/* PA1, PA0 */
}



/**
 *********************************************************************************************
 * @brief		Initialize UART.
 *
 * @return		none
 *********************************************************************************************
 */
static void init_uart(void)
{
	/* port setting(PK0, PK1) */
	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTK, ENABLE);
	TSB_PK->FR1 |= ((1 << 0) | (1 << 1U));
	TSB_PK->IE |= (1 << 0U);	/* PK0:RXD */
	TSB_PK->CR |= (1 << 1U);	/* PK1:TXD */

	/* UART setting(UART0) */
	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_UART0, ENABLE);

	TSB_UART0->SWRST = 0x2U;
	TSB_UART0->SWRST = 0x1U;

	while(TSB_UART0->SWRST==  0x80)
	{
	}

	TSB_UART0->BRD = UART_BRD_INIT;
	TSB_UART0->CLK = UART_CLK_INIT;
	TSB_UART0->CR0 = UART_CR0_INIT;
	TSB_UART0->CR1 = UART_CR1_INIT;
	NVIC_EnableIRQ(INTSC0TX_IRQn);

	send_step = 0;
	send_wait_time = 0;
}



/**
 *********************************************************************************************
 * @brief		UART Send data setting.
 *
 * @return		none
 *********************************************************************************************
 */
const uint8_t	tSendVEch[] = 			"ch";
const uint8_t	tSendCarrier[] = 		"LAg";
const uint8_t	tSendDead[] =			"fbh^C";
const uint8_t	tSendGate[] =			"Q[gANeBu";
const uint8_t	tSendshunt[] =			"ʒuo";
const uint8_t	tSendvdc[] =			"VDCd";
const uint8_t	tSendtemp[] =			"INVx";
const uint8_t	tSendphasevolt[] =		"U,V,Wd";
const uint8_t	tSenddac[] =			"DACgp";
const uint8_t	tSendinamp[] =			"Avgp";
const uint8_t	tSenddirection[] =		"]";
const uint8_t	tSendmodul[] =			"ϒ";
const uint8_t	tSendspeedchk[] =		"]w";


const uint8_t	tSendsunt2[] =			"Vg";
const uint8_t	tSendspwm[] =			"SPWM ";
const uint8_t	tSendno[] =				"";
const uint8_t	tSendyes[] =			"L";
const uint8_t	tSendtempsymbol[] =		"";
const uint8_t	tSendphase2[] =			"2ϒ";
const uint8_t	tSendphase3[] =			"3ϒ";
const uint8_t	tSendvolume[] =			"{[SW";
const uint8_t	tSendpush[] =			"vbVSW";
const uint8_t	tSendemg_DC[] =			"Over Vdc";
const uint8_t	tSendemg_I[] =			"Current detect error";
const uint8_t	tSendemg_S[] =			"Over current(soft)";
const uint8_t	tSendemg_H[] =			"Over current(hard)";

const uint8_t	tSendemg_Err[] = 		"Error";

static void send_data_set(void)
{
	uint8_t		i, digit, wait_send;
	float32_t	VE_value;
	uint32_t	set_value, set_value_mem;

	if(	(send_state == 0)		&&
		(send_wait_time == 0)	){
		txsize = 0;
		digit = 0;
		wait_send = 0;

#if defined(__CONTROL_MOTOR_CH1)
		vector_t* const _motor = &Motor_ch1;
		TSB_VE_TypeDef* pVE = TSB_VE1;

		switch(send_step){
			case 0:																/* CH */
				for(i = 0; i < sizeof(tSendVEch); i++){
					sendbuffer[txsize] = tSendVEch[i];
					txsize++;
				}

				sendbuffer[txsize] = 'C';
				txsize++;
				sendbuffer[txsize] = 'H';
				txsize++;
				sendbuffer[txsize] = '1';
				txsize++;
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 1:																/* LAg */
				for(i = 0; i < sizeof(tSendCarrier); i++){
					sendbuffer[txsize] = tSendCarrier[i];
					txsize++;
				}

				set_value = (uint32_t)(1000000 / cPWMPRD_CH1);
				set_value_mem = set_value;
				while(set_value_mem != 0){
					set_value_mem = set_value_mem / 10;
					digit++;
				}
				number_send_data_set(set_value, digit, 0);
				sendbuffer[txsize] = 'H';
				txsize++;
				sendbuffer[txsize] = 'z';
				txsize++;
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 2:																/* fbh^C */
				for(i = 0; i < sizeof(tSendDead); i++){
					sendbuffer[txsize] = tSendDead[i];
					txsize++;
				}

				set_value = (uint32_t)(cDEADTIME_CH1 * 10);
				number_send_data_set(set_value, 2, 1);
				sendbuffer[txsize] = 'u';
				txsize++;
				sendbuffer[txsize] = 's';
				txsize++;
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 3:																/* Q[gANeBu */
				for(i = 0; i < sizeof(tSendGate); i++){
					sendbuffer[txsize] = tSendGate[i];
					txsize++;
				}

				if(cPOLH_CH1 == 0){
					sendbuffer[txsize] = 'L';
					txsize++;
				}
				else{
					sendbuffer[txsize] = 'H';
					txsize++;
				}

				sendbuffer[txsize] = '/';
				txsize++;

				if(cPOLL_CH1 == 0){
					sendbuffer[txsize] = 'L';
					txsize++;
				}
				else{
					sendbuffer[txsize] = 'H';
					txsize++;
				}
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 4:																/* ʒuo */
				for(i = 0; i < sizeof(tSendshunt); i++){
					sendbuffer[txsize] = tSendshunt[i];
					txsize++;
				}

				if(cSHUNT_TYPE_CH1 == 1){
					sendbuffer[txsize] = '1';
					txsize++;
					for(i = 0; i < sizeof(tSendsunt2); i++){
						sendbuffer[txsize] = tSendsunt2[i];
						txsize++;
					}

					if(_motor->usr.com_user.spwm == 0){
						sendbuffer[txsize] = '(';
						txsize++;
						for(i = 0; i < sizeof(tSendspwm); i++){
							sendbuffer[txsize] = tSendspwm[i];
							txsize++;
						}
						for(i = 0; i < sizeof(tSendno); i++){
							sendbuffer[txsize] = tSendno[i];
							txsize++;
						}
						sendbuffer[txsize] = ')';
						txsize++;
					}
					else{
						sendbuffer[txsize] = '(';
						txsize++;
						for(i = 0; i < sizeof(tSendspwm); i++){
							sendbuffer[txsize] = tSendspwm[i];
							txsize++;
						}
						for(i = 0; i < sizeof(tSendyes); i++){
							sendbuffer[txsize] = tSendyes[i];
							txsize++;
						}
						sendbuffer[txsize] = ')';
						txsize++;
					}
				}
				else{
					sendbuffer[txsize] = '3';
					txsize++;
					for(i = 0; i < sizeof(tSendsunt2); i++){
						sendbuffer[txsize] = tSendsunt2[i];
						txsize++;
					}
				}
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 5:																/* VDCd */
				for(i = 0; i < sizeof(tSendvdc); i++){
					sendbuffer[txsize] = tSendvdc[i];
					txsize++;
				}

				VE_value = (float32_t)(_motor->drv.Vdc * cV_MAX_CH1 / FIX_15);
				set_value = (uint32_t)(VE_value * 10);
				set_value_mem = set_value;
				while(set_value_mem != 0){
					set_value_mem = set_value_mem / 10;
					digit++;
				}
				number_send_data_set(set_value, digit, 1);
				sendbuffer[txsize] = 'V';
				txsize++;
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 6:																/* INVx */
				for(i = 0; i < sizeof(tSendtemp); i++){
					sendbuffer[txsize] = tSendtemp[i];
					txsize++;
				}

				if( cTEMP_ERR == temp ){
					for(i = 0; i < sizeof(tSendemg_Err); i++){					/* set "Error"	*/
						sendbuffer[txsize] = tSendemg_Err[i];
						txsize++;
					}
				}
				else{
					if( temp < 0 ){
						sendbuffer[txsize] = '-';
						txsize++;
						temp *= -1;
					}
					else{
						/* do nothing	*/
					}
					set_value = (uint32_t)temp;

					set_value_mem = set_value;
					while(set_value_mem != 0){
						set_value_mem = set_value_mem / 10;
						digit++;
					}
					number_send_data_set(set_value, digit, 1);
					for(i = 0; i < sizeof(tSendtempsymbol); i++){
						sendbuffer[txsize] = tSendtempsymbol[i];
						txsize++;
					}
				}
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 7:																/* U,V,Wd */
				for(i = 0; i < sizeof(tSendphasevolt); i++){
					sendbuffer[txsize] = tSendphasevolt[i];
					txsize++;
				}

				VE_value = (float32_t)(pVE->IAADC * 5) / 65535;

				set_value = (uint32_t)(VE_value  * 100);
				set_value_mem = set_value;
				while(set_value_mem != 0){
					set_value_mem = set_value_mem / 10;
					digit++;
				}
				number_send_data_set(set_value, digit, 2);
				sendbuffer[txsize] = 'V';
				txsize++;
				sendbuffer[txsize] = ',';
				txsize++;
				digit = 0;

				VE_value = (float32_t)(pVE->IBADC * 5) / 65535;

				set_value = (uint32_t)(VE_value  * 100);
				set_value_mem = set_value;
				while(set_value_mem != 0){
					set_value_mem = set_value_mem / 10;
					digit++;
				}
				number_send_data_set(set_value, digit, 2);
				sendbuffer[txsize] = 'V';
				txsize++;
				sendbuffer[txsize] = ',';
				txsize++;
				digit = 0;

				VE_value = (float32_t)(pVE->ICADC * 5) / 65535;

				set_value = (uint32_t)(VE_value  * 100);
				set_value_mem = set_value;
				while(set_value_mem != 0){
					set_value_mem = set_value_mem / 10;
					digit++;
				}
				number_send_data_set(set_value, digit, 2);
				sendbuffer[txsize] = 'V';
				txsize++;
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 8:																/* DACgp */
				for(i = 0; i < sizeof(tSenddac); i++){
					sendbuffer[txsize] = tSenddac[i];
					txsize++;
				}

#if defined(__USE_DAC)
				for(i = 0; i < sizeof(tSendyes); i++){
					sendbuffer[txsize] = tSendyes[i];
					txsize++;
				}
#else
				for(i = 0; i < sizeof(tSendno); i++){
					sendbuffer[txsize] = tSendno[i];
					txsize++;
				}
#endif
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 9:																/* Avgp */
				for(i = 0; i < sizeof(tSendinamp); i++){
					sendbuffer[txsize] = tSendinamp[i];
					txsize++;
				}

#if defined(__USE_INAMP_CH1)
				for(i = 0; i < sizeof(tSendyes); i++){
					sendbuffer[txsize] = tSendyes[i];
					txsize++;
				}
#else
				for(i = 0; i < sizeof(tSendno); i++){
					sendbuffer[txsize] = tSendno[i];
					txsize++;
				}
#endif
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 10:															/* ] */
				for(i = 0; i < sizeof(tSenddirection); i++){
					sendbuffer[txsize] = tSenddirection[i];
					txsize++;
				}

				if(rote_dir == cROTATION_CW){
					sendbuffer[txsize] = 'C';
					txsize++;
					sendbuffer[txsize] = 'W';
					txsize++;
				}
				else{
					sendbuffer[txsize] = 'C';
					txsize++;
					sendbuffer[txsize] = 'C';
					txsize++;
					sendbuffer[txsize] = 'W';
					txsize++;
				}
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 11:															/* ϒ */
				for(i = 0; i < sizeof(tSendmodul); i++){
					sendbuffer[txsize] = tSendmodul[i];
					txsize++;
				}

				if(_motor->usr.com_user.modul == c2PHASE){
					for(i = 0; i < sizeof(tSendphase2); i++){
						sendbuffer[txsize] = tSendphase2[i];
						txsize++;
					}
				}
				else{
					for(i = 0; i < sizeof(tSendphase3); i++){
						sendbuffer[txsize] = tSendphase3[i];
						txsize++;
					}
				}
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 12:															/* ]w */
				for(i = 0; i < sizeof(tSendspeedchk); i++){
					sendbuffer[txsize] = tSendspeedchk[i];
					txsize++;
				}

				if(speed_ctr_mode == cSPDCTL_VR){
					for(i = 0; i < sizeof(tSendvolume); i++){
						sendbuffer[txsize] = tSendvolume[i];
						txsize++;
					}
				}
				else{
					for(i = 0; i < sizeof(tSendpush); i++){
						sendbuffer[txsize] = tSendpush[i];
						txsize++;
					}
				}
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 13:															/* x\ */
				if(start_motor == 1){
					if(_motor->stage.main == cEmergency){
						sendbuffer[txsize] = 'E';
						txsize++;
						sendbuffer[txsize] = 'M';
						txsize++;
						sendbuffer[txsize] = 'G';
						txsize++;
						sendbuffer[txsize] = '_';
						txsize++;
						sendbuffer[txsize] = 'S';
						txsize++;
						sendbuffer[txsize] = ':';
						txsize++;
						if(_motor->drv.state.flg.emg_DC == 1){
							for(i = 0; i < sizeof(tSendemg_DC); i++){
								sendbuffer[txsize] = tSendemg_DC[i];
								txsize++;
							}
						}
						else if(_motor->drv.state.flg.emg_I == 1){
							for(i = 0; i < sizeof(tSendemg_I); i++){
								sendbuffer[txsize] = tSendemg_I[i];
								txsize++;
							}
						}
						else if(_motor->drv.state.flg.emg_S == 1){
							for(i = 0; i < sizeof(tSendemg_S); i++){
								sendbuffer[txsize] = tSendemg_S[i];
								txsize++;
							}
						}
						
						else if(_motor->drv.state.flg.emg_H == 1){
							for(i = 0; i < sizeof(tSendemg_H); i++){
								sendbuffer[txsize] = tSendemg_H[i];
								txsize++;
							}
						}
						else{
						}
					}
					else{
						if(target_spd1 != 0){
							if(target_spd1 < 0){
								sendbuffer[txsize] = '-';
								txsize++;
								set_value = (uint32_t)(-target_spd1);
							}
							else{
								set_value = (uint32_t)(target_spd1);
							}
							set_value_mem = set_value;
							while(set_value_mem != 0){
								set_value_mem = set_value_mem / 10;
								digit++;
							}
							number_send_data_set(target_spd1, digit, 0);
						}
						else{
							sendbuffer[txsize] = '0';
							txsize++;
						}
						sendbuffer[txsize] = 'H';
						txsize++;
						sendbuffer[txsize] = 'z';
						txsize++;
					}
					sendbuffer[txsize] = 0x0d;
					txsize++;
					sendbuffer[txsize] = 0x0a;
					txsize++;
					send_wait_time = UART_SEND_WAIT;
				}
				else{
					wait_send = 1;
				}
				break;

			default:
				send_step = 0;
				break;
		}
#endif

#if defined(__CONTROL_MOTOR_CH0)
		vector_t* const _motor = &Motor_ch0;
		TSB_VE_TypeDef* pVE = TSB_VE0;

		switch(send_step){
			case 0:																/* CH */
				for(i = 0; i < sizeof(tSendVEch); i++){
					sendbuffer[txsize] = tSendVEch[i];
					txsize++;
				}

				sendbuffer[txsize] = 'C';
				txsize++;
				sendbuffer[txsize] = 'H';
				txsize++;
				sendbuffer[txsize] = '0';
				txsize++;
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 1:																/* LAg */
				for(i = 0; i < sizeof(tSendCarrier); i++){
					sendbuffer[txsize] = tSendCarrier[i];
					txsize++;
				}

				set_value = (uint32_t)(1000000 / cPWMPRD_CH0);
				set_value_mem = set_value;
				while(set_value_mem != 0){
					set_value_mem = set_value_mem / 10;
					digit++;
				}
				number_send_data_set(set_value, digit, 0);
				sendbuffer[txsize] = 'H';
				txsize++;
				sendbuffer[txsize] = 'z';
				txsize++;
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 2:																/* fbh^C */
				for(i = 0; i < sizeof(tSendDead); i++){
					sendbuffer[txsize] = tSendDead[i];
					txsize++;
				}

				set_value = (uint32_t)(cDEADTIME_CH0 * 10);
				number_send_data_set(set_value, 2, 1);
				sendbuffer[txsize] = 'u';
				txsize++;
				sendbuffer[txsize] = 's';
				txsize++;
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 3:																/* Q[gANeBu */
				for(i = 0; i < sizeof(tSendGate); i++){
					sendbuffer[txsize] = tSendGate[i];
					txsize++;
				}

				if(cPOLH_CH0 == 0){
					sendbuffer[txsize] = 'L';
					txsize++;
				}
				else{
					sendbuffer[txsize] = 'H';
					txsize++;
				}

				sendbuffer[txsize] = '/';
				txsize++;

				if(cPOLL_CH0 == 0){
					sendbuffer[txsize] = 'L';
					txsize++;
				}
				else{
					sendbuffer[txsize] = 'H';
					txsize++;
				}
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 4:																/* ʒuo */
				for(i = 0; i < sizeof(tSendshunt); i++){
					sendbuffer[txsize] = tSendshunt[i];
					txsize++;
				}

				if(cSHUNT_TYPE_CH0 == 1){
					sendbuffer[txsize] = '1';
					txsize++;
					for(i = 0; i < sizeof(tSendsunt2); i++){
						sendbuffer[txsize] = tSendsunt2[i];
						txsize++;
					}

					if(_motor->usr.com_user.spwm == 0){
						sendbuffer[txsize] = '(';
						txsize++;
						for(i = 0; i < sizeof(tSendspwm); i++){
							sendbuffer[txsize] = tSendspwm[i];
							txsize++;
						}
						for(i = 0; i < sizeof(tSendno); i++){
							sendbuffer[txsize] = tSendno[i];
							txsize++;
						}
						sendbuffer[txsize] = ')';
						txsize++;
					}
					else{
						sendbuffer[txsize] = '(';
						txsize++;
						for(i = 0; i < sizeof(tSendspwm); i++){
							sendbuffer[txsize] = tSendspwm[i];
							txsize++;
						}
						for(i = 0; i < sizeof(tSendyes); i++){
							sendbuffer[txsize] = tSendyes[i];
							txsize++;
						}
						sendbuffer[txsize] = ')';
						txsize++;
					}
				}
				else{
					sendbuffer[txsize] = '3';
					txsize++;
					for(i = 0; i < sizeof(tSendsunt2); i++){
						sendbuffer[txsize] = tSendsunt2[i];
						txsize++;
					}
				}
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 5:																/* VDCd */
				for(i = 0; i < sizeof(tSendvdc); i++){
					sendbuffer[txsize] = tSendvdc[i];
					txsize++;
				}

				VE_value = (float32_t)(_motor->drv.Vdc * cV_MAX_CH0 / FIX_15);
				set_value = (uint32_t)(VE_value * 10);
				set_value_mem = set_value;
				while(set_value_mem != 0){
					set_value_mem = set_value_mem / 10;
					digit++;
				}
				number_send_data_set(set_value, digit, 1);
				sendbuffer[txsize] = 'V';
				txsize++;
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 6:																/* INVx */
				for(i = 0; i < sizeof(tSendtemp); i++){
					sendbuffer[txsize] = tSendtemp[i];
					txsize++;
				}

				if( cTEMP_ERR == temp ){
					for(i = 0; i < sizeof(tSendemg_Err); i++){					/* set "Error"	*/
						sendbuffer[txsize] = tSendemg_Err[i];
						txsize++;
					}
				}
				else{
					if( temp < 0 ){
						sendbuffer[txsize] = '-';
						txsize++;
						temp *= -1;
					}
					else{
						/* do nothing	*/
					}
					set_value = (uint32_t)temp;

					set_value_mem = set_value;
					while(set_value_mem != 0){
						set_value_mem = set_value_mem / 10;
						digit++;
					}
					number_send_data_set(set_value, digit, 1);
					for(i = 0; i < sizeof(tSendtempsymbol); i++){
						sendbuffer[txsize] = tSendtempsymbol[i];
						txsize++;
					}
				}
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 7:																/* U,V,Wd */
				for(i = 0; i < sizeof(tSendphasevolt); i++){
					sendbuffer[txsize] = tSendphasevolt[i];
					txsize++;
				}

				VE_value = (float32_t)(pVE->IAADC * 5) / 65535;

				set_value = (uint32_t)(VE_value  * 100);
				set_value_mem = set_value;
				while(set_value_mem != 0){
					set_value_mem = set_value_mem / 10;
					digit++;
				}
				number_send_data_set(set_value, digit, 2);
				sendbuffer[txsize] = 'V';
				txsize++;
				sendbuffer[txsize] = ',';
				txsize++;
				digit = 0;

				VE_value = (float32_t)(pVE->IBADC * 5) / 65535;

				set_value = (uint32_t)(VE_value  * 100);
				set_value_mem = set_value;
				while(set_value_mem != 0){
					set_value_mem = set_value_mem / 10;
					digit++;
				}
				number_send_data_set(set_value, digit, 2);
				sendbuffer[txsize] = 'V';
				txsize++;
				sendbuffer[txsize] = ',';
				txsize++;
				digit = 0;

				VE_value = (float32_t)(pVE->ICADC * 5) / 65535;

				set_value = (uint32_t)(VE_value  * 100);
				set_value_mem = set_value;
				while(set_value_mem != 0){
					set_value_mem = set_value_mem / 10;
					digit++;
				}
				number_send_data_set(set_value, digit, 2);
				sendbuffer[txsize] = 'V';
				txsize++;
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 8:																/* DACgp */
				for(i = 0; i < sizeof(tSenddac); i++){
					sendbuffer[txsize] = tSenddac[i];
					txsize++;
				}

#if defined(__USE_DAC)
				for(i = 0; i < sizeof(tSendyes); i++){
					sendbuffer[txsize] = tSendyes[i];
					txsize++;
				}
#else
				for(i = 0; i < sizeof(tSendno); i++){
					sendbuffer[txsize] = tSendno[i];
					txsize++;
				}
#endif
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 9:																/* Avgp */
				for(i = 0; i < sizeof(tSendinamp); i++){
					sendbuffer[txsize] = tSendinamp[i];
					txsize++;
				}

#if defined(__USE_INAMP_CH0)
				for(i = 0; i < sizeof(tSendyes); i++){
					sendbuffer[txsize] = tSendyes[i];
					txsize++;
				}
#else
				for(i = 0; i < sizeof(tSendno); i++){
					sendbuffer[txsize] = tSendno[i];
					txsize++;
				}
#endif
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 10:															/* ] */
				for(i = 0; i < sizeof(tSenddirection); i++){
					sendbuffer[txsize] = tSenddirection[i];
					txsize++;
				}

				if(rote_dir == cROTATION_CW){
					sendbuffer[txsize] = 'C';
					txsize++;
					sendbuffer[txsize] = 'W';
					txsize++;
				}
				else{
					sendbuffer[txsize] = 'C';
					txsize++;
					sendbuffer[txsize] = 'C';
					txsize++;
					sendbuffer[txsize] = 'W';
					txsize++;
				}
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 11:															/* ϒ */
				for(i = 0; i < sizeof(tSendmodul); i++){
					sendbuffer[txsize] = tSendmodul[i];
					txsize++;
				}

				if(_motor->usr.com_user.modul == c2PHASE){
					for(i = 0; i < sizeof(tSendphase2); i++){
						sendbuffer[txsize] = tSendphase2[i];
						txsize++;
					}
				}
				else{
					for(i = 0; i < sizeof(tSendphase3); i++){
						sendbuffer[txsize] = tSendphase3[i];
						txsize++;
					}
				}
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 12:															/* ]w */
				for(i = 0; i < sizeof(tSendspeedchk); i++){
					sendbuffer[txsize] = tSendspeedchk[i];
					txsize++;
				}

				if(speed_ctr_mode == cSPDCTL_VR){
					for(i = 0; i < sizeof(tSendvolume); i++){
						sendbuffer[txsize] = tSendvolume[i];
						txsize++;
					}
				}
				else{
					for(i = 0; i < sizeof(tSendpush); i++){
						sendbuffer[txsize] = tSendpush[i];
						txsize++;
					}
				}
				sendbuffer[txsize] = 0x0d;
				txsize++;
				sendbuffer[txsize] = 0x0a;
				txsize++;
				send_step++;
				break;

			case 13:															/* x\ */
				if(start_motor == 1){
					if(_motor->stage.main == cEmergency){
						sendbuffer[txsize] = 'E';
						txsize++;
						sendbuffer[txsize] = 'M';
						txsize++;
						sendbuffer[txsize] = 'G';
						txsize++;
						sendbuffer[txsize] = '_';
						txsize++;
						sendbuffer[txsize] = 'S';
						txsize++;
						sendbuffer[txsize] = ':';
						txsize++;
						if(_motor->drv.state.flg.emg_DC == 1){
							for(i = 0; i < sizeof(tSendemg_DC); i++){
								sendbuffer[txsize] = tSendemg_DC[i];
								txsize++;
							}
						}
						else if(_motor->drv.state.flg.emg_I == 1){
							for(i = 0; i < sizeof(tSendemg_I); i++){
								sendbuffer[txsize] = tSendemg_I[i];
								txsize++;
							}
						}
						else if(_motor->drv.state.flg.emg_S == 1){
							for(i = 0; i < sizeof(tSendemg_S); i++){
								sendbuffer[txsize] = tSendemg_S[i];
								txsize++;
							}
						}
						
						else if(_motor->drv.state.flg.emg_H == 1){
							for(i = 0; i < sizeof(tSendemg_H); i++){
								sendbuffer[txsize] = tSendemg_H[i];
								txsize++;
							}
						}
						else{
						}
					}
					else{
						if(target_spd0 != 0){
							if(target_spd0 < 0){
								sendbuffer[txsize] = '-';
								txsize++;
								set_value = (uint32_t)(-target_spd0);
							}
							else{
								set_value = (uint32_t)(target_spd0);
							}
							set_value_mem = set_value;
							while(set_value_mem != 0){
								set_value_mem = set_value_mem / 10;
								digit++;
							}
							number_send_data_set(set_value, digit, 0);
						}
						else{
							sendbuffer[txsize] = '0';
							txsize++;
						}
						sendbuffer[txsize] = 'H';
						txsize++;
						sendbuffer[txsize] = 'z';
						txsize++;
					}
					sendbuffer[txsize] = 0x0d;
					txsize++;
					sendbuffer[txsize] = 0x0a;
					txsize++;
					send_wait_time = UART_SEND_WAIT;
				}
				else{
					wait_send = 1;
				}
				break;

			default:
				send_step = 0;
				break;
		}
#endif

		if(wait_send == 0){
			send_state = 1;
			txpnt = 0;
			TSB_UART0->DR = sendbuffer[txpnt];
			TSB_UART0->TRANS = UART_TXE_ENABLE;
		}
	}
	else{
		if(send_wait_time > 0){
			send_wait_time--;
		}
	}
}



/**
 *********************************************************************************************
 * @brief		UART Send data setting of nuber.
 *
 * @return		none
 *********************************************************************************************
 */
static void number_send_data_set(uint32_t _num, uint8_t _digit, uint8_t _decimal)
{
	uint8_t		i, pnt;
	uint8_t		number_buffer[20];

	i = _digit;

	while(i > 0)
	{
		pnt = (uint8_t)(i - 1);
		number_buffer[pnt] = (uint8_t)(0x30 + (_num % 10));
		_num = (uint32_t)(_num / 10);
		i--;
	}

	for(i = 0; i < _digit; i++)
	{
		if(i == (_digit - _decimal)){
			sendbuffer[txsize] = '.';
			txsize++;
		}
		sendbuffer[txsize] = number_buffer[i];
		txsize++;
	}
}



/**
 *********************************************************************************************
 * @brief		Transmission completion interrupt.
 *
 * @return		none
 *********************************************************************************************
 */
void INT_uart0tx(void)
{
	TSB_UART0->TRANS = UART_TXE_DISABLE;
	if((TSB_UART0->ERR & UART_ERR_CHECK) == 0)
	{
		txpnt++;
		if(txpnt >= txsize)
		{
			txpnt = 0;
			send_state = 0;
			TSB_UART0->SR = UART_TX_FLAG_CLR;
		}
		else
		{
			TSB_UART0->DR = sendbuffer[txpnt];
			TSB_UART0->TRANS = UART_TXE_ENABLE;
		}
	}
	else
	{
		TSB_UART0->ERR = UART_ERR_CLR;
		TSB_UART0->SR = UART_TX_FLAG_CLR;
		send_state = 0;
	}
}



/**
 ********************************************************************************************
 * @brief		Initialize NBD.
 *
 * @return		none
 ********************************************************************************************
 */
void init_NBD(void)
{
	/* Port */
	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTK, ENABLE);
	CG_SetFsysSupplyPeriphA(CG_FC_PERIPH_PORTL, ENABLE);

	/* NBDSYNC (PK4) */
	TSB_PK->FR6 |= (1 << 4U);
	TSB_PK->IE |= (1 << 4U);

	/* NBDDATA0,1,2,3, NBDCLK (PL0-4) */
	TSB_PL->FR6 |= ((1 << 4U) | (1 << 3U) | (1 << 2U) | (1 << 1U) | (1 << 0U));
	TSB_PL->CR |= ((1 << 3U) | (1 << 2U) | (1 << 1U) | (1 << 0U));
	TSB_PL->IE |= ((1 << 4U) | (1 << 3U) | (1 << 2U) | (1 << 1U) | (1 << 0U));

	/* Initialize */
	TSB_NBD->CR1 = 0xf9;
	TSB_NBD->CR0 |= 0x01;

}

/**
 ********************************************************************************************
 * @brief		UiOutDataStart by VE.
 *
 * @param[in]	uint8_t	_intch	.
 *
 * @return		none
 ********************************************************************************************
 */
void UiOutDataStartByVE(uint8_t	_intch)
{
	dac_t* const pdac = &dac;

	if (_intch == pdac->motch)		/* Dac output ch = call intve ch? */
	{
		uint16_t* const pdacout = DacData;
		vector_t* pmotor = 0;
		TSB_VE_TypeDef* pVE = 0;

		if (pdac->motch == 0)
		{
		#if defined(__CONTROL_MOTOR_CH0)
			pmotor = &Motor_ch0;
			pVE = TSB_VE0;
		#endif
		}
		else
		{
		#if defined(__CONTROL_MOTOR_CH1)
			pmotor = &Motor_ch1;
			pVE = TSB_VE1;
		#endif
		}	
	
		switch (pdac->select)
		{
			case 0://KESP1
				*(pdacout + 0) += 1;
				*(pdacout + 1) += 2;
				*(pdacout + 2) += 4;
				*(pdacout + 3) += 8;
				//*(pdacout + 0) = (pVE->IA						>> (16 - pdac->datsft0)) ^ 0x8000;
				//*(pdacout + 1) = (pVE->IB						>> (16 - pdac->datsft0)) ^ 0x8000;
				//*(pdacout + 2) = (pVE->IC						>> (16 - pdac->datsft0)) ^ 0x8000;
				//*(pdacout + 3) = pmotor->drv.theta.half[1];
				break;
			case 1:
				*(pdacout + 0) = (pmotor->drv.Id_ref			<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.Id				<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.Iq_ref			<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 3) = (pmotor->drv.Iq				<< pdac->datsft0) ^ 0x8000;
				break;
			case 2:
				*(pdacout + 0) = (pmotor->drv.omega_com.half[1]	<< pdac->datsft2) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.omega.half[1] 	<< pdac->datsft2) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.omega_dev 		<< pdac->datsft2) ^ 0x8000;
				*(pdacout + 3) = (pmotor->drv.Iq_ref			<< pdac->datsft0) ^ 0x8000;
				break;
			case 3:
				*(pdacout + 0) = (pVE->IA						>> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.Iq_ref			<< pdac->datsft0)		 ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.Id_ref			<< pdac->datsft0)		 ^ 0x8000;
				*(pdacout + 3) = (pmotor->drv.omega.half[1]		<< pdac->datsft2)		 ^ 0x8000;
				break;
			case 4:
				*(pdacout + 0) = (pVE->IA						>> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.omega.half[1]		<< pdac->datsft2) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.Iq_ref			<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 3) = pmotor->drv.theta.half[1];
				break;
			case 5:
				*(pdacout + 0) = pmotor->drv.Vdc_ave.half[1]	<< pdac->datsft1;
				*(pdacout + 1) = pmotor->drv.Vdq_ave.half[1]	<< pdac->datsft1;
				*(pdacout + 2) = (pmotor->drv.Id_ref			<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 3) = (pmotor->drv.Iq_ref			<< pdac->datsft0) ^ 0x8000;
				break;
			case 6:
				*(pdacout + 0) = (pmotor->drv.omega.half[1]		<< pdac->datsft2) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.Iq				<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.Ed				<< pdac->datsft1) ^ 0x8000;
				*(pdacout + 3) = (pmotor->drv.theta.half[1]);
				break;
			case 7:
				*(pdacout + 0) = (pVE->VD						>> (16 - pdac->datsft1)) ^ 0x8000;
				*(pdacout + 1) = (pVE->IA						>> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.omega.half[1]		<< pdac->datsft2)		 ^ 0x8000;
				*(pdacout + 3) = pmotor->drv.theta.half[1];
				break;
			case 8:
				*(pdacout + 0) = (pVE->IAADC);
				*(pdacout + 1) = (pVE->IBADC);
				*(pdacout + 2) = (pVE->ICADC);
				*(pdacout + 3) = (pVE->VDC);
				break;
			case 9:
				*(pdacout + 0) = (pmotor->drv.Ia				<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.Ib				<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 2) = (pmotor->drv.Ic				<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 3) = (pmotor->para.err_ovc			<< pdac->datsft0) ^ 0x8000;
				break;
			case 10:
			#if defined(__USE_ENCODER_CH0) || defined(__USE_ENCODER_CH1)
				*(pdacout + 0) = (pmotor->drv.theta.half[1]);
				*(pdacout + 1) = (pmotor->drv.theta_enc >> 16);
				*(pdacout + 2) = (pmotor->drv.theta_com);
				*(pdacout + 3) = (pVE->IA >>					(16 - pdac->datsft0)) ^ 0x8000;
			#endif
				break;
			case 11:
			#if defined(__USE_ENCODER_CH0) || defined(__USE_ENCODER_CH1)
				*(pdacout + 0) = (pmotor->drv.theta.half[1]);
				*(pdacout + 1) = (pmotor->drv.theta_enc >> 16);
				*(pdacout + 2) = (pmotor->drv.theta_com);
				*(pdacout + 3) = pmotor->drv.EnCnt;
			#endif
				break;
			case 12:
				*(pdacout + 0) = (pmotor->drv.Id				<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 1) = (pmotor->drv.Iq				<< pdac->datsft0) ^ 0x8000;
				*(pdacout + 2) = (pVE->IA						>> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 3) = pVE->SECTOR					<< 11;
				break;
			case 13:
				*(pdacout + 0) = (pVE->TMPREG0 >> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 1) = (pVE->TMPREG1 >> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 2) = (pVE->TMPREG2 >> (16 - pdac->datsft0)) ^ 0x8000;
				*(pdacout + 3) = pmotor->drv.theta.half[1];
				break;
			default:
				*(pdacout + 0) += 0x0100;
				*(pdacout + 1) += 0x0200;
				*(pdacout + 2) += 0x0300;
				*(pdacout + 3) += 0x0400;
				break;
		}

		DacWriteStart(DAC_SIO_CH);

	}
}


/******************************** END OF FILE *********************************/
